import inspect
from importlib import import_module
from netengine.exceptions import NetEngineError

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django_hstore.fields import DictionaryField

from nodeshot.core.base.models import BaseDate, BaseOrdered
from nodeshot.core.base.managers import HStoreNodeshotManager
from nodeshot.networking.net.models import *
from nodeshot.networking.net.models.choices import DEVICE_STATUS
from nodeshot.core.base.utils import now

from ..settings import settings, NETENGINE_BACKENDS

if 'nodeshot.networking.hardware' in settings.INSTALLED_APPS:
    HARDWARE_INSTALLED = True
    from nodeshot.networking.hardware.models import DeviceModel, Manufacturer, DeviceToModelRel
else:
    HARDWARE_INSTALLED = False


class DeviceConnector(BaseDate, BaseOrdered):
    """
    DeviceConnector Model
    """
    backend = models.CharField(_('backend'), max_length=128,
                              choices=NETENGINE_BACKENDS,
                              help_text=_('select the operating system / protocol to use to retrieve info from device'))
    node = models.ForeignKey('nodes.Node', verbose_name=_('node'))
    host = models.CharField(_('host'), max_length=128)
    config = DictionaryField(_('config'), blank=True, null=True,
                            help_text=_('backend specific parameters, eg: username/password (SSH), community (SNMP)'))
    port = models.IntegerField(_('port'), blank=True, null=True,
                               help_text=_('leave blank to use the default port for the protocol in use'))
    store = models.BooleanField(_('store in DB?'),
                                default=True,
                                help_text=_('is adviced to store read-only credentials only'))
    device = models.ForeignKey(Device, verbose_name=_('device'),
                               blank=True, null=True,
                               help_text=_('leave blank, will be created automatically'))

    # django manager
    objects = HStoreNodeshotManager()

    __netengine = None
    __backend_class = None

    class Meta:
        ordering = ["order"]
        app_label = 'connectors'
        verbose_name = _('device connector')
        verbose_name_plural = _('device connectors')

    def __unicode__(self):
        if self.host:
            return self.host
        else:
            return _(u'Unsaved Device Connector')

    def save(self, *args, **kwargs):
        """
        Custom save does the following:
            * strip trailing whitespace from host attribute
            * create device and all other related objects
            * store connection config in DB if store attribute is True
        """
        self.host = self.host.strip()

        if not self.id:
            self.device = self.__create_device()

        if self.store is True:
            super(DeviceConnector, self).save(*args, **kwargs)

    def clean(self, *args, **kwargs):
        """ validation """
        self._validate_backend()
        self._validate_config()
        self._validate_netengine()
        self._validate_duplicates()

    @property
    def REQUIRED_CONFIG_KEYS(self):
        return self._get_netengine_arguments(required=True)

    @property
    def AVAILABLE_CONFIG_KEYS(self):
        return self._get_netengine_arguments()

    @property
    def backend_class(self):
        """
        returns python netengine backend class, importing it if needed
        """
        if not self.backend:
            return None

        if not self.__backend_class:
            self.__backend_class = self._get_netengine_backend()

        return self.__backend_class

    @property
    def netengine(self):
        """ access netengine instance """
        # return None if no backend chosen yet
        if not self.backend:
            return None

        # init instance of the netengine backend if not already done
        if not self.__netengine:
            NetengineBackend = self.backend_class
            arguments = self._build_netengine_arguments()

            self.__netengine = NetengineBackend(**arguments)

        # return netengine instance
        return self.__netengine

    def _validate_backend(self):
        """ ensure backend string representation is correct """
        try:
            self.backend_class
        # if we get an import error the specified path is wrong
        except (ImportError, AttributeError) as e:
            raise ValidationError(_('No valid backend found, got the following python exception: "%s"') % e)

    def _validate_config(self):
        """ ensure REQUIRED_CONFIG_KEYS are filled """
        # exit if no backend specified
        if not self.backend:
            return
        # exit if no required config keys
        if len(self.REQUIRED_CONFIG_KEYS) < 1:
            return

        self.config = self.config or {}  # default to empty dict of no config
        required_keys_set = set(self.REQUIRED_CONFIG_KEYS)
        config_keys_set = set(self.config.keys())
        missing_required_keys = required_keys_set - config_keys_set
        unrecognized_keys = config_keys_set - required_keys_set

        # if any missing required key raise ValidationError
        if len(missing_required_keys) > 0:
            # converts list in comma separated string
            missing_keys_string = ', '.join(missing_required_keys)
            # django error
            raise ValidationError(_('Missing required config keys: "%s"') % missing_keys_string)
        elif len(unrecognized_keys) > 0:
            # converts list in comma separated string
            unrecognized_keys_string = ', '.join(unrecognized_keys)
            # django error
            raise ValidationError(_('Unrecognized config keys: "%s"') % unrecognized_keys_string)

    def _validate_netengine(self):
        """
        call netengine validate() method
        verifies connection parameters are correct
        """
        if self.backend:
            try:
                self.netengine.validate()
            except NetEngineError as e:
                raise ValidationError(e)

    def _validate_duplicates(self):
        """
        Ensure we're not creating a device that already exists
        Runs only when the DeviceConnector object is created, not when is updated
        """
        # if connector is being created right now
        if not self.id:
            duplicates = []
            self.netengine_dict = self.netengine.to_dict()
            # loop over interfaces and check mac address
            for interface in self.netengine_dict['interfaces']:
                # avoid checking twice for the same interface (often ifconfig returns duplicates)
                if interface['mac_address'] in duplicates:
                    continue
                # check in DB
                if Interface.objects.filter(mac__iexact=interface['mac_address']).count() > 0:
                    duplicates.append(interface['mac_address'])

            # if we have duplicates raise validation error
            if len(duplicates) > 0:
                mac_address_string = ', '.join(duplicates)
                raise ValidationError(_('interfaces with the following mac addresses already exist: %s') % mac_address_string)

    def _get_netengine_arguments(self, required=False):
        """
        returns list of available config params
        returns list of required config params if required is True
        for internal use only
        """
        # inspect netengine class
        backend_class = self._get_netengine_backend()
        argspec = inspect.getargspec(backend_class.__init__)
        # store args
        args = argspec.args
        # remove known arguments
        for argument_name in ['self', 'host', 'port']:
            args.remove(argument_name)

        if required:
            # list of default values
            default_values = list(argspec.defaults)
            # always remove last default value, which is port number
            default_values = default_values[0:-1]

            # remove an amount of arguments equals to number of default values, starting from right
            args = args[0:len(args)-len(default_values)]

        return args

    def _get_netengine_backend(self):
        """
        returns the netengine backend specified in self.backend
        for internal use only
        """
        # extract backend class name, eg: AirOS or OpenWRT
        backend_class_name = self.backend.split('.')[-1]
        # convert to lowercase to get the path
        backend_path = self.backend.lower()
        # import module by its path
        module = import_module(backend_path)
        # get netengine backend class
        BackendClass = getattr(module, backend_class_name)

        return BackendClass

    def _build_netengine_arguments(self):
        """
        returns a python dictionary representing arguments
        that will be passed to a netengine backend
        for internal use only
        """
        arguments = {
            "host": self.host
        }

        if self.config is not None:
            for key, value in self.config.iteritems():
                arguments[key] = value

        if self.port:
            arguments["port"] = self.port

        return arguments

    def get_auto_order_queryset(self):
        """
        Overriding a method of BaseOrdered Abstract Model
        """
        return self.__class__.objects.filter(device=self.device)

    def __create_device(self):
        """
        creates device, internal use only
        """
        # retrieve netengine dictionary from memory or from network
        device_dict = getattr(self, 'netengine_dict', self.netengine.to_dict())
        device = Device()
        device.node_id = self.node_id
        device.name = device_dict['name']
        device.type = device_dict['type']
        device.status = DEVICE_STATUS.get('reachable')
        device.os = device_dict['os']
        device.os_version = device_dict['os_version']
        # this is the first time the device is seen by the system because we are just adding it
        device.first_seen = now()
        # and is also the latest
        device.last_seen = now()
        device.full_clean()
        device.save()

        # add routing protocols
        for routing_protocol in device_dict['routing_protocols']:
            # retrieve routing protocol from DB
            try:
                rp = RoutingProtocol.objects.filter(
                    name__iexact=routing_protocol['name'],
                    version__iexact=routing_protocol['version']
                )[0]
            # create if doesn't exist yet
            except IndexError:
                rp = RoutingProtocol(
                    name=routing_protocol['name'],
                    version=routing_protocol['version']
                )
                rp.full_clean()
                rp.save()
            # add to device
            device.routing_protocols.add(rp)

        for interface in device_dict['interfaces']:
            interface_object = False
            vap_object = False
            # create interface depending on type
            if interface['type'] == 'ethernet':
                interface_object = Ethernet(**{
                    'device': device,
                    'name': interface['name'],
                    'mac': interface['mac_address'],
                    'mtu': interface['mtu'],
                    'standard': interface['standard'],
                    'duplex': interface['duplex'],
                    'tx_rate': interface['tx_rate'],
                    'rx_rate': interface['rx_rate']
                })
            elif interface['type'] == 'wireless':
                interface_object = Wireless(**{
                    'device': device,
                    'name': interface['name'],
                    'mac': interface['mac_address'],
                    'mtu': interface['mtu'],
                    'mode': interface['mode'],
                    'standard': interface['standard'],
                    'channel': interface['channel'],
                    'channel_width': interface['channel_width'],
                    'output_power': interface['output_power'],
                    'dbm': interface['dbm'],
                    'noise': interface['noise'],
                    'tx_rate': interface['tx_rate'],
                    'rx_rate': interface['rx_rate']
                })

                for vap in interface['vap']:
                    vap_object = Vap(
                        essid=vap['essid'],
                        bssid=vap['bssid'],
                        encryption=vap['encryption']
                    )

            if interface_object:
                interface_object.full_clean()
                interface_object.save()

                if vap_object:
                    vap_object.interface = interface_object
                    vap_object.full_clean()
                    vap_object.save()

                for ip in interface['ip']:
                    ip_object = Ip(**{
                        'interface': interface_object,
                        'address': ip['address'],
                    })
                    ip_object.full_clean()
                    ip_object.save()

        if HARDWARE_INSTALLED:
            # try getting device model from db
            try:
                device_model = DeviceModel.objects.filter(name__iexact=device_dict['model'])[0]
            # if it does not exist create it
            except IndexError as e:
                # try getting manufacturer from DB
                try:
                    manufacturer = Manufacturer.objects.filter(name__iexact=device_dict['manufacturer'])[0]
                # or create
                except IndexError as e:
                    manufacturer = Manufacturer(name=device_dict['manufacturer'])
                    manufacturer.full_clean()
                    manufacturer.save()

                device_model = DeviceModel(
                    manufacturer=manufacturer,
                    name=device_dict['model']
                )
                device_model.ram = device_dict['RAM_total']

            device_model.full_clean()
            device_model.save()

            # create relation between device model and device
            rel = DeviceToModelRel(device=device, model=device_model)
            rel.full_clean()
            rel.save()

        return device



from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from nodeshot.core.base.models import BaseAccessLevel

from choices import DEVICE_STATUS, DEVICE_STATUS_CHOICES, DEVICE_TYPES_CHOICES

from django_hstore.fields import DictionaryField, ReferencesField
from nodeshot.core.base.managers import HStoreGeoAccessLevelManager as DeviceManager


class Device(BaseAccessLevel):
    """
    Device Model
    Represents a network device
    eg: an outdoor point-to-point wifi device, a BGP router, a server, and so on
    """
    name = models.CharField(_('name'), max_length=50)
    node = models.ForeignKey('nodes.Node', verbose_name=_('node'))
    type = models.CharField(_('type'), max_length=50, choices=DEVICE_TYPES_CHOICES)
    status = models.SmallIntegerField(_('status'), max_length=2, choices=DEVICE_STATUS_CHOICES, default=DEVICE_STATUS.get('unknown'))
    
    # geographic data
    location = models.PointField(_('location'), blank=True, null=True,
                                 help_text=_("""specify device coordinates (if different from node);
                                    defaults to node coordinates if node is a point,
                                    otherwise if node is a geometry it will default to che centroid of the geometry"""))
    elev = models.FloatField(_('elevation'), blank=True, null=True)
    
    # device specific
    routing_protocols = models.ManyToManyField('net.RoutingProtocol', blank=True)
    
    os = models.CharField(_('operating system'), max_length=128, blank=True, null=True)
    os_version = models.CharField(_('operating system version'), max_length=128, blank=True, null=True)
    
    first_seen = models.DateTimeField(_('first time seen on'), blank=True, null=True, default=None)
    last_seen = models.DateTimeField(_('last time seen on'), blank=True, null=True, default=None)
    
    # text
    description = models.CharField(_('description'), max_length=255, blank=True, null=True)
    notes = models.TextField(_('notes'), blank=True, null=True)
    
    # extra data
    data = DictionaryField(_('extra data'), null=True, blank=True,
                           help_text=_('store extra attributes in JSON string'))
    shortcuts = ReferencesField(null=True, blank=True)
    
    objects = DeviceManager()
    
    # list indicating if any other module has extended this model
    extended_by = []
    
    class Meta:
        app_label = 'net'
        
    def __unicode__(self):
        return '%s' % self.name
    
    def save(self, *args, **kwargs):
        """
        Custom save method does the following:
            * automatically inherit node coordinates and elevation
            * save shortcuts if HSTORE is enabled
        """
        custom_checks = kwargs.pop('custom_checks', True)
        
        super(Device, self).save(*args, **kwargs)
        
        if custom_checks is False:
            return
        
        changed = False
        
        if not self.location:
            self.location = self.node.point
            changed = True
        
        if not self.elev and self.node.elev:
            self.elev = self.node.elev
            changed = True
        
        original_user = self.shortcuts.get('user')
        
        self.shortcuts['user'] = self.node.user
        
        if original_user != self.shortcuts.get('user'):
            changed = True
        
        if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
            original_layer = self.shortcuts.get('layer')
            self.shortcuts['layer'] = self.node.layer
            
            if original_layer != self.shortcuts.get('layer'):
                changed = True
        
        if changed:
            self.save(custom_checks=False)
    
    @property
    def owner(self):
        if 'user' not in self.shortcuts:
            if self.node or self.node_id:
                self.save()
            else:
                raise Exception('Instance does not have a node set yet')
        return self.shortcuts['user']
    
    @property
    def layer(self):
        if 'nodeshot.core.layers' not in settings.INSTALLED_APPS:
            return False
        if 'layer' not in self.shortcuts:
            if self.node or self.node_id:
                self.save()
            else:
                raise Exception('Instance does not have a node set yet')
        return self.shortcuts['layer']
    
    if 'grappelli' in settings.INSTALLED_APPS:
        @staticmethod
        def autocomplete_search_fields():
            return ('name__icontains',)



from django.db import models


class UpdateCountsMixin(models.Model):
    """
    Updates node_rating_count record each time an
    Instance of an extended model is created or deleted
    """

    class Meta:
        abstract = True

    def update_count(self):
        """ this method needs to be overwritten """
        pass

    def save(self, *args, **kwargs):
        """ custom save method to update counts """
        # the following lines determines if the comment is being created or not
        # in case the comment exists the pk attribute is an int
        created = type(self.pk) is not int

        super(UpdateCountsMixin, self).save(*args, **kwargs)

        # this operation must be performed after the parent save
        if created:
            self.update_count()

    def delete(self, *args, **kwargs):
        """ custom delete method to update counts """
        super(UpdateCountsMixin, self).delete(*args, **kwargs)
        self.update_count()



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseDate
from . import ImageMixin, AntennaModel


class RadiationPattern(BaseDate, ImageMixin):
    """ Radiation Pattern of an Antenna Model """
    antenna_model = models.ForeignKey(AntennaModel)
    type = models.CharField(_('type'), max_length=30)
    image = models.ImageField(upload_to='antennas/radiation_patterns/', verbose_name=_('image'))
    
    def __unicode__(self):
        return _('radiation pattern for antenna model: %s' % self.antenna_model)
    
    class Meta:
        app_label= 'hardware'
        db_table = 'hardware_radiation_pattern'



from django.db import models
from django.utils.translation import ugettext_lazy as _

from . import Manufacturer


class MacPrefix(models.Model):
    """ Mac prefix of a Manufacturer """
    manufacturer = models.ForeignKey(Manufacturer, verbose_name=_('manufacturer'))
    prefix = models.CharField(_('mac address prefix'), max_length=8, unique=True)
    
    def __unicode__(self):
        return self.prefix
    
    class Meta:
        app_label= 'hardware'
        verbose_name = _('MAC Prefix')
        verbose_name_plural = _('MAC Prefixes')



from django.shortcuts import render
from nodeshot.ui.default.settings import TILESERVER_URL


def open311(request):
    context = {
        'TILESERVER_URL': TILESERVER_URL
    }
    return render(request,'open311/index.html', context)



from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _


class ImageMixin(models.Model):
    """
    Abstract model with few useful methods to display an image
    """
    
    image_width = 80
    
    class Meta:
        abstract = True
    
    def __unicode__(self, *args, **kwargs):
        return self.name
    
    def image_url(self):
        return '%s%s' % (settings.MEDIA_URL, self.image) 
    
    def image_img_tag(self):
        if self.image != '':
            return '<img src="%s" alt="" style="width:%spx" />' % (self.image_url(), self.image_width)
        else:
            return _('No image available')
    
    image_img_tag.allow_tags = True


# this app is dependant on "net"
from django.conf import settings
if 'nodeshot.networking.net' not in settings.INSTALLED_APPS:
    from nodeshot.core.base.exceptions import DependencyError
    raise DependencyError('nodeshot.networking.hardware depends on nodeshot.networking.net, which should be in settings.INSTALLED_APPS')


from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from .settings import ACL_DEFAULT_VALUE, ACL_DEFAULT_EDITABLE
from .choices import ACCESS_LEVELS
from .utils import choicify, now


class BaseShortcut(models.Model):
    """
    Abstract Model providing shortcuts to main manager methods
    Handy for shell prototyping
    """
    
    class Meta:
        abstract = True
        
    @classmethod
    def all(cls):
        return cls.objects.all()
    
    @classmethod
    def filter(cls, *args, **kwargs):
        return cls.objects.filter(*args, **kwargs)
    
    @classmethod
    def exclude(cls, *args, **kwargs):
        return cls.objects.exclude(*args, **kwargs)
    
    @classmethod
    def count(cls, *args, **kwargs):
        return cls.objects.count(*args, **kwargs)
    
    # --- custom methods, copying Rails :) --- #
    
    @classmethod
    def last(cls):
        return cls.objects.last()
    
    @classmethod
    def first(cls):
        return cls.objects.first()
    
    @classmethod
    def find(cls, pk):
        return cls.objects.find(pk)


class BaseDate(BaseShortcut):
    """
    Base Abstract Model that provides:
        * an added field that automatically sets the insert date
        * an updated field which updates itself automatically
    We don't use django's autoaddnow=True because that makes the fields not visible in the admin.
    """
    added = models.DateTimeField(_('created on'), default=now())
    updated = models.DateTimeField(_('updated on'), default=now())

    class Meta:
        abstract = True
    
    def save(self, *args, **kwargs):
        """
        automatically update updated date field
        """
        # auto fill updated field with current time unless explicitly disabled
        auto_update = kwargs.get('auto_update', True)
        if auto_update:
            self.updated = now()
        
        # remove eventual auto_update
        if 'auto_update' in kwargs:
            kwargs.pop('auto_update')
        
        super(BaseDate, self).save(*args, **kwargs)


class BaseAccessLevel(BaseDate):
    """
    Base Abstract Model that extends BaseDate and provides
    an additional field for access level management.
    The field default and editable attributes value can be
    overriden by adding some directives in the settings file.
    
    DEFAULT VALUE
        To edit the default value for the access_level field of a certain
        model you will have to add the following setting in your settings.py file:
    
        NODESHOT_ACL_{APP_NAME}_{MODEL_NAME}_DEFAULT = 'public'
    
        where {APP_NAME} is the uppercase name of an app like "NODES"
        and {MODEL_NAME} is the uppercase name of a model like "NODE"
        The values will have to be one of the possible values specified in
        "nodeshot.core.base.choices.ACCESS_LEVELS"
        The possible values are public or the id of the group saved in the database
        (default ones are 1 for registered, 2 for community and 3 for trusted)
        
        For the cases in which no setting is specified the fallback setting
        NODESHOT_ACL_DEFAULT_VALUE will be used.
    
    EDITABLE
        If you want to disable the possibility to edit the access_level field
        for a given model you will have to add the following settings in the settings.py file:
        
        NODESHOT_ACL_{APP_NAME}_{MODEL_NAME}_EDITABLE = False
        
        where {APP_NAME} is the uppercase name of an app like "NODES"
        and {MODEL_NAME} is the uppercase name of a model like "NODE"
        
        For the cases in which no setting is specified the fallback setting
        NODESHOT_ACL_DEFAULT_EDITABLE will be used.
    
    """
    access_level = models.SmallIntegerField(_('access level'), choices=choicify(ACCESS_LEVELS), default=ACCESS_LEVELS.get(ACL_DEFAULT_VALUE))
    
    class Meta:
        abstract = True
    
    def __init__(self, *args, **kwargs):
        """
        Determines default value for field "access_level" and determines if is editable
        In the case the field is not editable it won't show up at all
        """
        
        # {APP_NAME}_{MODEL_NAME}, eg: NODES_NODE
        app_descriptor = '%s_%s' % (self._meta.app_label.upper(), self._meta.object_name.upper())
        
        # looks up in settings.py
        # example: NODESHOT_ACL_NODES_NODE_DEFAULT
        # defaults to NODESHOT_ACL_DEFAULT_VALUE
        value = getattr(settings, 'NODESHOT_ACL_%s_DEFAULT' % app_descriptor, ACL_DEFAULT_VALUE)
        ACL_DEFAULT = ACCESS_LEVELS.get(value)
        
        # looks up in settings.py
        # example: NODESHOT_ACL_NODES_NODE_EDITABLE
        ACL_EDITABLE = getattr(settings, 'NODESHOT_ACL_%s_EDITABLE' % app_descriptor, ACL_DEFAULT_EDITABLE)
        
        # set "default" and "editable" attributes
        self._meta.get_field('access_level').default = ACL_DEFAULT
        self._meta.get_field('access_level').editable = ACL_EDITABLE
        
        # call super __init__
        super(BaseAccessLevel, self).__init__(*args, **kwargs)


class BaseOrdered(models.Model):
    """
    Ordered Model provides functions for ordering objects in the Django Admin
    """
    order = models.PositiveIntegerField(blank=True, help_text=_('Leave blank to set as last'))

    class Meta:
        ordering = ["order"]
        abstract = True
    
    def get_auto_order_queryset(self):
        return self.__class__.objects.all()
    
    def save(self, *args, **kwargs):
        """ if order left blank """
        if self.order == '' or self.order is None:
            try:
                self.order = self.get_auto_order_queryset().order_by("-order")[0].order + 1
            except IndexError:
                self.order = 0
        super(BaseOrdered, self).save()


class BaseOrderedACL(BaseOrdered, BaseAccessLevel):
    
    class Meta:
        ordering = ["order"]
        abstract = True



from django.conf import settings


HSTORE_SCHEMA = getattr(settings, 'NODESHOT_LAYERS_HSTORE_SCHEMA', None)
NODES_MINIMUM_DISTANCE = getattr(settings, 'NODESHOT_LAYERS_NODES_MINIMUM_DISTANCE', 0)
REVERSION_ENABLED = getattr(settings, 'NODESHOT_LAYERS_REVERSION_ENABLED', True)
TEXT_HTML = getattr(settings, 'NODESHOT_LAYERS_TEXT_HTML', True)


if HSTORE_SCHEMA:
    ADDITIONAL_LAYER_FIELDS = [field.get('name') for field in HSTORE_SCHEMA]
else:
    ADDITIONAL_LAYER_FIELDS = []



from django.http import Http404
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth import get_user_model
User = get_user_model()

from rest_framework import generics, authentication, permissions
from rest_framework.response import Response

from nodeshot.core.nodes.models import Node

from .serializers import *
from .models import Inward
from .settings import settings, INWARD_REQUIRE_AUTH


class ContactNode(generics.CreateAPIView):
    """
    Contact owner of specified node.

    Might require authentication depending on configuration.

    Name and email fields will be determined automatically if the user is authenticated.
    """
    authentication_classes = (authentication.SessionAuthentication,)
    permission_classes = (permissions.IsAuthenticated,) if INWARD_REQUIRE_AUTH else ()
    serializer_class = InwardMessageSerializer
    content_type = 'node'
    model = Node
    slug_field = 'slug'

    def is_contactable(self):
        return True

    def get_object(self, *args, **kwargs):
        try:
            self.recipient = self.model.objects.get(**{ self.slug_field: kwargs.get(self.slug_field, False) })
        except Node.DoesNotExist:
            raise Http404('Not Found')

        self.object = Inward()

    def get(self, request, *args, **kwargs):
        try:
            self.get_object(**kwargs)
        except Http404:
            return Response({ 'detail': _('Not Found') }, status=404)

        if not self.is_contactable():
            return Response({ 'detail': _('This resource cannot be contacted') }, status=400)

        return Response({ 'detail': _('Method Not Allowed') }, status=405)

    def post(self, request, *args, **kwargs):
        """
        Contact node owner.
        """
        try:
            self.get_object(**kwargs)
        except Http404:
            return Response({ 'detail': _('Not Found') }, status=404)

        if not self.is_contactable():
            return Response({ 'detail': _('This resource cannot be contacted') }, status=400)

        content_type = ContentType.objects.only('id', 'model').get(model=self.content_type)

        # shortcut
        data = request.DATA

        # init inward message
        message = Inward()
        # required fields
        message.content_type = content_type
        message.object_id = self.recipient.id
        message.message = data.get('message')

        # user info if authenticated
        if request.user.is_authenticated():
            message.user = request.user
        else:
            message.from_name = data.get('from_name')
            message.from_email = data.get('from_email')

        # additional user info
        message.ip = request.META.get('REMOTE_ADDR', '')
        message.user_agent = request.META.get('HTTP_USER_AGENT', '')
        message.accept_language = request.META.get('HTTP_ACCEPT_LANGUAGE', '')

        try:
            message.full_clean()
        except ValidationError, e:
            return Response(e.message_dict, status=400)

        message.save()

        if message.status >= 1:
            return Response({ 'details': _('Message sent successfully') }, status=201)
        else:
            return Response({ 'details': _('Something went wrong. The email was not sent.') }, status=500)

contact_node = ContactNode.as_view()


class ContactUser(ContactNode):
    """
    Contact specified user.

    Might require authentication depending on configuration.

    Name and email fields will be determined automatically if the user is authenticated.
    """
    content_type = User.__name__.lower()
    model = User
    slug_field = 'username'

contact_user = ContactUser.as_view()


if 'nodeshot.core.layers' in settings.INSTALLED_APPS:

    from nodeshot.core.layers.models import Layer

    class ContactLayer(ContactNode):
        """
        Contact mantainers of specified Layer.

        Might require authentication depending on configuration.

        Name and email fields will be determined automatically if the user is authenticated.
        """
        content_type = 'layer'
        model = Layer
        slug_field = 'slug'

        def is_contactable(self):
            return bool(self.recipient.email or self.recipient.mantainers.count() > 1)

    contact_layer = ContactLayer.as_view()



import socket
import time

from django.db import models
from django.db.models import Q
from django.core.validators import MaxLengthValidator, MinLengthValidator
from django.core.exceptions import ValidationError
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.utils.translation import ugettext_lazy as _
from django.utils.html import strip_tags
from django.contrib.auth import get_user_model

from nodeshot.core.base.models import BaseDate
from nodeshot.core.base.fields import MultiSelectField
from nodeshot.core.base.utils import now
from nodeshot.core.nodes.models import Node

from .choices import *  # noqa
from ..settings import settings, OUTWARD_SCHEDULING, OUTWARD_MINLENGTH, OUTWARD_MAXLENGTH, OUTWARD_HTML, OUTWARD_STEP, OUTWARD_DELAY


class Outward(BaseDate):
    """
    This is a tool that can be used to send out newsletters or important communications to your community.
    It's aimed to be useful and flexible.
    """
    status = models.IntegerField(_('status'), choices=OUTWARD_STATUS_CHOICES,
                                 default=OUTWARD_STATUS.get('draft'))
    subject = models.CharField(_('subject'), max_length=50)
    message = models.TextField(_('message'), validators=[
        MinLengthValidator(OUTWARD_MINLENGTH),
        MaxLengthValidator(OUTWARD_MAXLENGTH)
    ])
    is_scheduled = models.SmallIntegerField(_('schedule sending'),
                                            choices=SCHEDULE_CHOICES,
                                            default=0)
    scheduled_date = models.DateField(_('scheduled date'), blank=True, null=True)
    scheduled_time = models.CharField(_('scheduled time'), max_length=20,
                                      choices=OUTWARD_SCHEDULING,
                                      default=OUTWARD_SCHEDULING[0][0],
                                      blank=True)
    is_filtered = models.SmallIntegerField(_('recipient filtering'),
                                           choices=FILTERING_CHOICES,
                                           default=0)
    filters = MultiSelectField(max_length=255, choices=FILTER_CHOICES,
                               blank=True,
                               help_text=_('specify recipient filters'))
    groups = MultiSelectField(max_length=255, choices=GROUPS,
                              default=DEFAULT_GROUPS,
                              blank=True,
                              help_text=_('filter specific groups of users'))

    if 'nodeshot.core.layers' in settings.INSTALLED_APPS:
        layers = models.ManyToManyField('layers.Layer', verbose_name=_('layers'), blank=True)

    users = models.ManyToManyField(settings.AUTH_USER_MODEL, verbose_name=_('users'), blank=True)

    class Meta:
        verbose_name = _('Outward message')
        verbose_name_plural = _('Outward messages')
        app_label = 'mailing'
        ordering = ['status']

    def __unicode__(self):
        return '%s' % self.subject

    def get_recipients(self):
        """
        Determine recipients depending on selected filtering which can be either:
            * group based
            * layer based
            * user based

        Choosing "group" and "layer" filtering together has the effect of sending the message
        only to users for which the following conditions are both true:
            * have a node assigned to one of the selected layers
            * are part of any of the specified groups (eg: registered, community, trusted)

        The user based filtering has instead the effect of translating in an **OR** query. Here's a practical example:
        if selecting "group" and "user" filtering the message will be sent to all the users for which ANY of the following conditions is true:
            * are part of any of the specified groups (eg: registered, community, trusted)
            * selected users
        """
        # user model
        User = get_user_model()

        # prepare email list
        emails = []

        # the following code is a bit ugly. Considering the titanic amount of work required to build all
        # the cools functionalities that I have in my mind, I can't be bothered to waste time on making it nicer right now.
        # if you have ideas on how to improve it to make it cleaner and less cluttered, please join in
        # this method has unit tests written for it, therefore if you try to change it be sure to check unit tests do not fail after your changes
        # python manage.py test mailing

        # send to all case
        if not self.is_filtered:
            # retrieve only email DB column of all active users
            users = User.objects.filter(is_active=True).only('email')
            # loop over users list
            for user in users:
                # add email to the recipient list if not already there
                if user.email not in emails:
                    emails += [user.email]
        else:
            # selected users
            if FILTERS.get('users') in self.filters:
                # retrieve selected users
                users = self.users.all().only('email')
                # loop over selected users
                for user in users:
                    # add email to the recipient list if not already there
                    if user.email not in emails:
                        emails += [user.email]

            # Q is a django object for "complex" filtering queries (not that complex in this case)
            # init empty Q object that will be needed in case of group filtering
            q = Q()
            q2 = Q()

            # if group filtering is checked
            if FILTERS.get('groups') in self.filters:
                # loop over each group
                for group in self.groups:
                    # if not superusers
                    if group != '0':
                        # add the group to the Q object
                        # this means that the query will look for users of that specific group
                        q = q | Q(groups=int(group))
                        q2 = q2 | Q(user__groups=int(group))
                    else:
                        # this must be done manually because superusers is not a group but an attribute of the User model
                        q = q | Q(is_superuser=True)
                        q2 = q2 | Q(user__is_superuser=True)

            # plus users must be active
            q = q & Q(is_active=True)

            # if layer filtering is checked
            if FILTERS.get('layers') in self.filters:
                # retrieve non-external layers
                layers = self.layers.all().only('id')
                # init empty q3
                q3 = Q()
                # loop over layers to form q3 object
                for layer in layers:
                    q3 = q3 | Q(layer=layer)
                # q2: user group if present
                # q3: layers
                # retrieve nodes
                nodes = Node.objects.filter(q2 & q3)
                # loop over nodes of a layer and get their email
                for node in nodes:
                    # add email to the recipient list if not already there
                    if node.user.email not in emails:
                        emails += [node.user.email]
            # else if group filterins is checked but not layers
            elif FILTERS.get('groups') in self.filters and not FILTERS.get('layers') in self.filters:
                # retrieve only email DB column of all active users
                users = User.objects.filter(q).only('email')
                # loop over users list
                for user in users:
                    # add email to the recipient list if not already there
                    if user.email not in emails:
                        emails += [user.email]
        return emails

    def send(self):
        """
        Sends the email to the recipients
        """
        # if it has already been sent don't send again
        if self.status is OUTWARD_STATUS.get('sent'):
            return False
        # determine recipients
        recipients = self.get_recipients()
        # init empty list that will contain django's email objects
        emails = []

        # prepare text plain if necessary
        if OUTWARD_HTML:
            # store plain text in var
            html_content = self.message
            # set EmailClass to EmailMultiAlternatives
            EmailClass = EmailMultiAlternatives
        else:
            EmailClass = EmailMessage

        # default message is plain text
        message = strip_tags(self.message)

        # loop over recipients and fill "emails" list
        for recipient in recipients:
            msg = EmailClass(
                # subject
                self.subject,
                # message
                message,
                # from
                settings.DEFAULT_FROM_EMAIL,
                # to
                [recipient],
            )
            if OUTWARD_HTML:
                msg.attach_alternative(html_content, "text/html")
            # prepare email object
            emails.append(msg)

        # try sending email
        try:
            # counter will count how many emails have been sent
            counter = 0
            for email in emails:
                # if step reached
                if counter == OUTWARD_STEP:
                    # reset counter
                    counter = 0
                    # sleep
                    time.sleep(OUTWARD_DELAY)
                # send email
                email.send()
                # increase counter
                counter += 1
        # if error (connection refused, SMTP down)
        except socket.error as e:
            # log the error
            from logging import error
            error('nodeshot.core.mailing.models.outward.send(): %s' % e)
            # set status of the instance as "error"
            self.status = OUTWARD_STATUS.get('error')
        # change status
        self.status = OUTWARD_STATUS.get('sent')
        # save
        self.save()

    def save(self, *args, **kwargs):
        """
        Custom save method
        """
        # change status to scheduled if necessary
        if self.is_scheduled and self.status is not OUTWARD_STATUS.get('scheduled'):
            self.status = OUTWARD_STATUS.get('scheduled')

        # call super.save()
        super(Outward, self).save(*args, **kwargs)

    def clean(self, *args, **kwargs):
        """
        Custom validation
        """
        if self.is_scheduled is 1 and (self.scheduled_date == '' or self.scheduled_date is None
                                       or self.scheduled_time == '' or self.scheduled_time is None):
            raise ValidationError(_('If message is scheduled both fields "scheduled date" and "scheduled time" must be specified'))

        if self.is_scheduled is 1 and self.scheduled_date < now().date():
            raise ValidationError(_('The scheduled date is set to a past date'))

        if self.is_filtered is 1 and (len(self.filters) < 1 or self.filters == [''] or
                                      self.filters == [u''] or self.filters == '' or self.filters is None):
            raise ValidationError(_('If "recipient filtering" is active one of the filtering options should be selected'))

        if self.is_filtered is 1 and FILTERS.get('groups') in self.filters and\
           (len(self.groups) < 1 or self.groups == [''] or self.groups == [u''] or self.groups == '' or self.groups is None):
            raise ValidationError(_('If group filtering is active at least one group of users should be selected'))

        # TODO: unfortunately layers and users can't be validated easily because they are a many2many field



from django.contrib import admin
from django.utils.translation import ugettext as _

from nodeshot.core.layers.admin import  LayerAdmin
from nodeshot.core.nodes.admin import NodeAdmin

from .models import LayerExternal, NodeExternal
from .settings import settings
from .tasks import synchronize_external_layers


class LayerExternalInline(admin.StackedInline):
    model = LayerExternal
    fk_name = 'layer'

    if 'grappelli' in settings.INSTALLED_APPS:
        inline_classes = ('grp-collapse grp-open',)

    def get_formset(self, request, obj=None, **kwargs):
        """
        Load Synchronizer schema to display specific fields in admin
        """
        if obj is not None:
            try:
                # this is enough to load the new schema
                obj.external
            except LayerExternal.DoesNotExist:
                pass
        return super(LayerExternalInline, self).get_formset(request, obj=None, **kwargs)


def synchronize_action(self, request, queryset):
    synchronized = []
    ignored = []
    for layer in queryset:
        if layer.is_external:
            synchronize_external_layers.apply_async([layer.slug])
            synchronized.append(layer)
        else:
            ignored.append(layer)

    if len(synchronized) == 1:
        synced_message_bit = _("1 layer was synchronized.")
    else:
        synced_message_bit = _("%s layers were synchronized.") % len(synchronized)

    if len(ignored) == 1:
        ignored_message_bit = _("1 layer was ignored because not flagged as external.")
    elif len(ignored) > 1:
        ignored_message_bit = _("%s layers were ignored because not flagged as external.") % len(ignored)
    else:
        ignored_message_bit = ""

    self.message_user(request, "%s %s" % (synced_message_bit, ignored_message_bit), fail_silently=True)
synchronize_action.short_description = _("Synchronize layer (external only)")


# add inline to LayerAdmin
LayerAdmin.inlines.append(LayerExternalInline)
# custom admin template
LayerAdmin.change_form_template = 'admin/layer_change_form.html'
# add synchronize action
LayerAdmin.synchronize_action = synchronize_action
LayerAdmin.actions.append('synchronize_action')


class NodeExternalInline(admin.StackedInline):
    model = NodeExternal
    fk_name = 'node'
    extra = 0

NodeAdmin.inlines.append(NodeExternalInline)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.base.models import BaseAccessLevel
from nodeshot.core.base.managers import AccessLevelManager

from . import Category
from .choices import SERVICE_STATUS


class Service(BaseAccessLevel):
    """
    Service Model
    Describes a service, eg: ftp, storage, proxy, ecc.
    """
    device = models.ForeignKey('net.Device', verbose_name=_('device'))
    name = models.CharField(_('name'), max_length=30)
    category = models.ForeignKey(Category, verbose_name=_('category') )
    description = models.TextField(_('description'), blank=True, null=True)
    documentation_url = models.URLField(_('documentation url'), blank=True, null=True)
    status = models.SmallIntegerField(_('status'), choices=SERVICE_STATUS)
    is_published = models.BooleanField(_('published'), default=True)
    
    objects = AccessLevelManager()
    
    class Meta:
        app_label = 'services'
        verbose_name = _('service')
        verbose_name_plural = _('services')
    
    def __unicode__(self):
        return '%s' % self.name


from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.interop.sync.views',
    url(r'^admin/layer-external-reload-schema/(?P<pk>\d+)/$',
        'layer_external_reload_schema',
        name='layer_external_reload_schema'),
)


from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.nodes.models import Node


class NodeExternal(models.Model):
    """
    External Node info, extend 'Node' with additional data
    """
    node = models.OneToOneField(Node, verbose_name=_('node'), parent_link=True, related_name='external')
    external_id = models.CharField(_('external id'), blank=True, max_length=255,
                                   help_text=_("""ID of this node on the external layer, might be a hash or an integer
                                               or whatever other format the external application uses to store IDs"""))

    class Meta:
        app_label = 'sync'
        db_table = 'nodes_external'
        verbose_name = _('external node')
        verbose_name_plural = _('external node info')

    def __unicode__(self):
        return '%s additional data' % self.node.name


# ------ Signals ------ #


from django.dispatch import receiver
from django.db.models.signals import pre_delete, post_save

from ..tasks import push_changes_to_external_layers


@receiver(post_save, sender=Node)
def save_external_nodes(sender, **kwargs):
    """ sync by creating nodes in external layers when needed """
    node = kwargs['instance']
    operation = 'add' if kwargs['created'] is True else 'change'

    if node.layer.is_external is False or not hasattr(node.layer, 'external') or node.layer.external.synchronizer_path is None:
        return False

    push_changes_to_external_layers.delay(node=node, external_layer=node.layer.external, operation=operation)


@receiver(pre_delete, sender=Node)
def delete_external_nodes(sender, **kwargs):
    """ sync by deleting nodes from external layers when needed """
    node = kwargs['instance']

    if node.layer.is_external is False or not hasattr(node.layer, 'external') or node.layer.external.synchronizer_path is None:
        return False

    if hasattr(node, 'external') and node.external.external_id:
        push_changes_to_external_layers.delay(
            node=node.external.external_id,
            external_layer=node.layer.external,
            operation='delete'
        )



from __future__ import absolute_import

import requests
from requests.exceptions import RequestException
import simplejson as json

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from .base import BaseSynchronizer, GenericGisSynchronizer


__all__ = ['NodeshotMixin', 'Nodeshot']


class NodeshotMixin(object):
    """
    Nodeshot synchronizer mixin
    RESTfrul translator type
    """
    SCHEMA = [
        {
            'name': 'layer_url',
            'class': 'CharField',
            'kwargs': {
                'max_length': 255,
                'help_text': _('URL of external layer, in the form of https://HOST/api/v1/layers/LAYER_NAME/')
            }
        },
        GenericGisSynchronizer.SCHEMA[1]  # verify ssl
    ]

    def get_nodes(self, class_name, params):
        prefix = self.config['layer_url']

        is_geojson = 'geojson' in class_name.lower()

        suffix = 'nodes/' if not is_geojson else 'nodes.geojson'
        # url from where to fetch nodes
        url = '%s%s' % (prefix, suffix)

        try:
            response = requests.get(url, params=params, verify=self.verify_ssl)
        except RequestException as e:
            return {
                'error': _('external layer not reachable'),
                'exception': str(e)
            }

        if response.status_code != 200:
            return {
                'error': _('HTTP request failed'),
                'exception': response.status_code
            }

        try:
            response.data = json.loads(response.content)
        except json.scanner.JSONDecodeError as e:
            return {
                'error': _('external layer is experiencing some issues because it returned invalid data'),
                'exception': str(e)
            }

        if 'nodes' in response.data:
            nodes = response.data['nodes']
        else:
            nodes = response.data

        # replace pagination links
        for direction in ['previous', 'next']:
            if direction in nodes and nodes[direction]:
                querystring = nodes[direction].split('?')[1]
                view_name = 'api_layer_nodes_geojson' if is_geojson else 'api_layer_nodes_list'
                path = reverse(view_name, args=[self.layer.slug])
                url = '{base_url}{path}?{querystring}'.format(
                    base_url=settings.SITE_URL,
                    path=path,
                    querystring=querystring
                )
                nodes[direction] = url

        return nodes


class Nodeshot(NodeshotMixin, BaseSynchronizer):
    """ Nodeshot synchronizer """
    pass



"""
nodeshot.networking.connectors unit tests
"""

import simplejson as json

from netengine.backends.ssh import OpenWRT
from netengine.exceptions import NetEngineError

from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse

from nodeshot.core.base.tests import BaseTestCase
from nodeshot.core.base.tests import user_fixtures
from nodeshot.networking.net.models import *

from .models import DeviceConnector
from . import settings

settings.NETENGINE_BACKENDS += [('netengine.backends.Dummy', 'Dummy')]


class ConnectorTest(BaseTestCase):

    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_routing_protocols.json',
        'test_devices.json',
        'test_interfaces.json',
        'test_ip_addresses.json'
    ]

    def setUp(self):
        self.client.login(username='admin', password='tester')
        self.c1 = DeviceConnector.objects.create(backend='netengine.backends.Dummy',
                                                 node_id=1,
                                                 host='127.0.0.1')

    def test_unicode(self):
        unicode(self.c1)

    def test_admin_changelist(self):
        url = reverse('admin:connectors_deviceconnector_changelist')
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_admin_change(self):
        url = reverse('admin:connectors_deviceconnector_change', args=[self.c1.id])
        response = self.client.get(url)
        self.assertEquals(response.status_code, 200)

    def test_saved_data(self):
        c1 = self.c1
        self.assertEquals(c1.device.name, 'dummy')
        self.assertEquals(c1.device.os, 'dummyOS')
        self.assertEquals(c1.device.interface_set.count(), 2)

        # ensure a new routing protocol has been created
        rp_count = RoutingProtocol.objects.filter(name='olsr', version='dummy version').count()
        self.assertEquals(rp_count, 1)

        # ensure vap interfaces have been created
        wireless = Wireless.objects.filter(device=c1.device)[0]
        self.assertEquals(wireless.vap_set.count(), 1)

    def test_get_netengine_backend(self):
        self.c1.backend = 'netengine.backends.ssh.OpenWRT'
        NetEngineBackend = self.c1._get_netengine_backend()
        self.assertTrue(NetEngineBackend is OpenWRT)

    def test_build_netengine_arguments(self):
        c1 = DeviceConnector(
            backend='netengine.backends.ssh.OpenWRT',
            host='127.0.0.1',
            config = {
                'username': 'root',
                'password':'test'
            }
        )
        arguments = c1._build_netengine_arguments()
        test_arguments = {
            'host': self.c1.host,
            'username': 'root',
            'password': 'test'
        }
        self.assertEqual(arguments, test_arguments)

        c1.port = 20200
        test_arguments['port'] = 20200
        arguments = c1._build_netengine_arguments()
        self.assertEqual(arguments, test_arguments)

        c1.config = {}
        c1.port = None
        arguments = c1._build_netengine_arguments()
        self.assertEqual(arguments, { 'host': self.c1.host })

    def test_get_netengine_arguments(self):
        c1 = DeviceConnector(
            backend='netengine.backends.ssh.OpenWRT',
            host='127.0.0.1',
            config = {
                'username': 'root',
                'password':'test'
            }
        )
        self.assertEqual(c1._get_netengine_arguments(), ['username', 'password'])
        self.assertEqual(c1.AVAILABLE_CONFIG_KEYS, ['username', 'password'])
        self.assertEqual(c1._get_netengine_arguments(required=True), ['username', 'password'])
        self.assertEqual(c1.REQUIRED_CONFIG_KEYS, ['username', 'password'])

        c2 = DeviceConnector()
        c2.backend = 'netengine.backends.snmp.AirOS'
        self.assertEqual(c2._get_netengine_arguments(), ['community', 'agent'])
        self.assertEqual(c2.AVAILABLE_CONFIG_KEYS, ['community', 'agent'])
        self.assertEqual(c2._get_netengine_arguments(required=True), [])
        self.assertEqual(c2.REQUIRED_CONFIG_KEYS, [])

    def test_validation_ok(self):
        self.c1.device.delete()
        self.c1.delete()
        c2 = DeviceConnector()
        c2.backend = 'netengine.backends.Dummy'
        c2.node_id = self.c1.node_id
        c2.host = '127.0.0.1'
        c2.full_clean()
        c2.save()

    def test_validate_backend(self):
        # wrong connector class
        c1 = DeviceConnector(backend='iam.very.wrong',
                             node_id=1,
                             host='127.0.0.1')
        try:
            c1._validate_backend()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertIn('iam.very.wrong', e.messages[0])
            self.assertEquals(len(e.messages), 1)

    def test_validate_config_missing_keys(self):
        # missing username and password
        c2 = DeviceConnector()
        c2.backend = 'netengine.backends.ssh.AirOS'
        c2.node_id = self.c1.node_id
        c2.host = '127.0.0.1'
        try:
            c2._validate_config()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertEquals(len(e.messages), 1)
            self.assertIn('username, password', e.messages[0])

        # missing password
        c2.config = { 'username': 'root' }
        try:
            c2._validate_config()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertEquals(len(e.messages), 1)
            self.assertIn('password', e.messages[0])

        # missing username
        c2.config = { 'password': 'test' }
        try:
            c2._validate_config()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertEquals(len(e.messages), 1)
            self.assertIn('username', e.messages[0])

        # nothing missing
        c2.config['username'] = 'test'
        c2._validate_config()

    def test_validate_config_unrecognized_keys(self):
        c2 = DeviceConnector()
        c2.backend = 'netengine.backends.ssh.AirOS'
        c2.node_id = self.c1.node_id
        c2.host = '127.0.0.1'

        # unrecoginzed parameter "wrong"
        c2.config = {
            'username': 'user',
            'password': 'test',
            'wrong': 'wrong'
        }
        try:
            c2._validate_config()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertEquals(len(e.messages), 1)
            self.assertIn('wrong', e.messages[0])

        # no unrecognized keys
        c2.config.pop('wrong')
        c2._validate_config()

    def test_validate_netengine(self):
        c1 = DeviceConnector(
            backend='netengine.backends.ssh.AirOS',
            host='192.168.1.254',
            config = {
                'username': 'user',
                'password':'test'
            }
        )
        # expect connection error
        with self.assertRaises(ValidationError):
            c1._validate_netengine()

    def test_validate_duplicates(self):
        dup = DeviceConnector(backend='netengine.backends.Dummy',
                              node_id=1,
                              host='127.0.0.1')

        try:
            dup._validate_duplicates()
            self.fail('ValidationError not raised')
        except ValidationError as e:
            self.assertEquals(len(e.messages), 1)
            self.assertIn('de:9f:db:30:c9:c4', e.messages[0])
            self.assertIn('de:9f:db:30:c9:c5', e.messages[0])



from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
from rest_framework.reverse import reverse

from nodeshot.core.nodes.models import Node, Image
from nodeshot.community.participation.models import Vote, Comment, Rating
from nodeshot.core.base.serializers import ExtraFieldSerializer

from .base import SERVICES
from .settings import settings, TYPE, STATUS


__all__ = [
    'ServiceRatingSerializer',
    'ServiceCommentSerializer',
    'ServiceNodeSerializer',
    'ServiceVoteSerializer',
    'ServiceListSerializer',
    'NodeRequestSerializer',
    'VoteRequestSerializer',
    'CommentRequestSerializer',
    'RatingRequestSerializer',
    'NodeRequestDetailSerializer',
    'NodeRequestListSerializer',
    'VoteRequestListSerializer',
    'CommentRequestListSerializer',
    'RatingRequestListSerializer',
]

RATING_CHOICES = [ n for n in range(1, 11) ]
VOTING_CHOICES = [ -1, 1 ]


class ServiceListSerializer(serializers.Serializer):
    """
    Open 311 service list
    """
    definition = serializers.SerializerMethodField('get_definition')
    metadata = serializers.SerializerMethodField('get_metadata')
    keywords = serializers.SerializerMethodField('get_keywords')
    group = serializers.SerializerMethodField('get_group')
    type = serializers.SerializerMethodField('get_type')
    service_code = serializers.SerializerMethodField('get_service_code')
    service_name = serializers.SerializerMethodField('get_service_name')
    description = serializers.SerializerMethodField('get_service_description')

    def __init__(self, *args, **kwargs):
        self.service_type = kwargs.pop('service_type','node')
        super(ServiceListSerializer, self).__init__(*args, **kwargs)

    def get_definition(self, obj):
        request = self.context['request']
        format = self.context['format']
        return reverse('api_service_definition_detail',
                       args=[self.service_type],
                       request=request,
                       format=format)

    def get_service_code(self, obj):
        return self.service_type

    def get_service_name(self, obj):
        return SERVICES[self.service_type]['name']

    def get_service_description(self, obj):
        return SERVICES[self.service_type]['description']

    def get_keywords(self, obj):
        return SERVICES[self.service_type]['keywords']

    def get_group(self,obj):
        return SERVICES[self.service_type]['group']

    def get_metadata(self,obj):
        """ Open311 metadata indicates whether there are custom attributes """
        return 'true'

    def get_type(self,obj):
        """ type setting - TODO explain """
        return TYPE

    class Meta:
        fields= (
            'service_code',
            'service_name',
            'description',
            'keywords',
            'group',
            'definition',
            'metadata',
            'type',
        )


class ServiceNodeSerializer(serializers.Serializer):
    """
    Service details
    """
    service_code = serializers.SerializerMethodField('get_service_code')
    attributes = serializers.SerializerMethodField('get_attributes')

    def get_service_code(self, obj):
        return 'node'

    def get_attributes(self, obj):
        return [
            # layer
            {
                'code': 'layer',
                'description': _('layer_slug'),
                'datatype': 'string',
                'datatype_description': _('Specify the slug of the layer in which you want to insert the node'),
                'order': 1,
                'required': True,
                'variable' : True
            },

            # name
            {
                'code': 'name',
                'description': _('name'),
                'datatype': 'string',
                'datatype_description': _('Name of the node you want to insert'),
                'order': 2,
                'required': True,
                'variable' : True
            },

            # lat
            {
                'code': 'lat',
                'description': _('latitude'),
                'datatype': 'string',
                'datatype_description': _('Latitude of node'),
                'order': 3,
                'required': True,
                'variable' : True
            },

            # long
            {
                'code': 'long',
                'description': _('longitude'),
                'datatype': 'string',
                'datatype_description': _('Longitude of node'),
                'order': 4,
                'required': True,
                'variable' : True
            },

            # address
            {
                'code': 'address',
                'description': _('address'),
                'datatype': 'string',
                'datatype_description': _('Address of node'),
                'order': 5,
                'required': False,
                'variable' : True
            },

            # elev (elevation)
            {
                'code': 'elev',
                'description': _('elevation'),
                'datatype': 'string',
                'datatype_description': _('Elevation of node'),
                'order': 6,
                'required': False,
                'variable' : True
            },

            # description
            {
                'code': 'description',
                'description': _('description'),
                'datatype': 'string',
                'datatype_description': _('Description of node'),
                'order': 7,
                'required': False,
                'variable' : True
            },

            # images
            {
                'code': 'images',
                'description': _('images'),
                'datatype': 'string',
                'datatype_description': _('Images related to node. A client may POST multiple files as multipart/form-data.\
                                          Requests return the URL for this images via the image_url field'),
                'order': 8,
                'required': False,
                'variable' : True
            }
        ]

    class Meta:
        fields = ('service_code', 'attributes')


class ServiceVoteSerializer(serializers.Serializer):
    """
    Service details
    """
    service_code = serializers.SerializerMethodField('get_service_code')
    attributes = serializers.SerializerMethodField('get_attributes')

    def get_service_code(self, obj):
        return 'vote'

    def get_attributes(self, obj):
        return [
            # node_id
            {
                'code': 'node_id',
                'description': _('Node id'),
                'datatype': 'string',
                'datatype_description': _('Specify for which node you want to insert the vote'),
                'order': 1,
                'required': True,
                'variable' : True

            },

            # vote
            {
                'code': 'vote',
                'description': _('vote on a node'),
                'datatype': 'string',
                'datatype_description': _('Vote 1 or -1 (Like/Dislike)'),
                'order': 2,
                'required': True,
                'variable' : True,
                'values' : [1,-1]
            },

        ]

    class Meta:
        fields = ('service_code', 'attributes')


class ServiceCommentSerializer(serializers.Serializer):
    """
    Service details
    """
    service_code = serializers.SerializerMethodField('get_service_code')
    attributes = serializers.SerializerMethodField('get_attributes')

    def get_service_code(self, obj):
        return 'comment'

    def get_attributes(self, obj):
        return [
            # node_id
            {
                'code': 'node_id',
                'description': _('Node id'),
                'datatype': 'string',
                'datatype_description': _('Specify for which node you want to insert the comment'),
                'order': 1,
                'required': True,
                'variable' : True
            },

            # vote
            {
                'code': 'comment',
                'description': _('comment on a node'),
                'datatype': 'string',
                'datatype_description': _('text of the comment'),
                'order': 2,
                'required': True,
                'variable' : True

            },

        ]

    class Meta:
        fields = ('service_code', 'attributes')


class ServiceRatingSerializer(serializers.Serializer):
    """
    Service details
    """
    service_code = serializers.SerializerMethodField('get_service_code')
    attributes = serializers.SerializerMethodField('get_attributes')

    def get_service_code(self, obj):
        return 'rate'

    def get_attributes(self, obj):
        return [
            # node_id
            {
                'code': 'node_id',
                'description': _('Node id'),
                'datatype': 'string',
                'datatype_description': _('Specify which node you want to rate'),
                'order': 1,
                'required': True,
                'variable' : True,
                'required': True

            },

            # rating
            {
                'code': 'rating',
                'description': _('rating of a node'),
                'datatype': 'string',
                'datatype_description': _('rate node from 1 to 10'),
                'order': 2,
                'required': True,
                'variable' : True,
                'values' : RATING_CHOICES
            },

        ]

    class Meta:
        fields = ('service_code', 'attributes')


class NodeRequestListSerializer(ExtraFieldSerializer):
    """
    Open 311 node request
    """
    service_request_id = serializers.SerializerMethodField('get_service_request_id')
    layer_slug= serializers.SerializerMethodField('get_layer_slug')
    details = serializers.SerializerMethodField('get_details')
    image_urls = serializers.SerializerMethodField('get_image_urls')
    requested_datetime = serializers.Field(source='added')
    updated_datetime = serializers.Field(source='updated')
    lat = serializers.CharField()
    long = serializers.CharField()
    image = serializers.ImageField()
    service_code = serializers.CharField()
    layer = serializers.CharField()

    def get_image_urls(self,obj):
        image_url =[]

        images = Image.objects.filter(node=obj.id)

        if images:
            for image in images:
                image_url.append('%s%s' % (settings.MEDIA_URL, image.file))
            return image_url
        else:
            return ""

    def get_layer_name(self, obj):
        if obj is None:
            return ""
        layer_name =  obj.layer
        return layer_name

    def get_layer_slug(self, obj):
        if obj is None:
            return ""
        layer_slug =  obj.layer.slug
        return layer_slug

    def get_service_request_id(self, obj):
        if obj is None:
            return ""
        service_request_id = 'node-%d' % obj.id
        return service_request_id

    def get_details(self, obj):
        if obj is None:
            return ""
        request = self.context['request']
        format = self.context['format']

        return reverse('api_service_request_detail',
                       args=['node',obj.id],
                       request=request,
                       format=format)

    class Meta:
        model = Node
        fields= ('service_request_id', 'slug', 'name', 'service_code', 'layer', 'layer_slug', 'status', 'geometry', 'name',
                 'description', 'requested_datetime', 'updated_datetime', 'image_urls','image',
                 'details', 'address', 'lat', 'long', )
        read_only_fields = ('geometry', 'id', 'status', 'is_published', 'access_level',
                            'data','notes','user','added','updated', 'slug', )
        non_native_fields = ( 'service_code', 'lat', 'long',
                                'elev', 'image', 'layer_slug', )


class NodeRequestDetailSerializer(NodeRequestListSerializer):
    """
    Open 311 node request
    """

    class Meta:
        model = Node
        fields= ('layer','layer_slug','slug','name','status', 'geometry', 'description', 'address',
                 'requested_datetime', 'updated_datetime', 'image_urls',)


class NodeRequestSerializer(serializers.ModelSerializer):
    """
    Open 311 node request
    """

    class Meta:
        model = Node


class VoteRequestListSerializer(ExtraFieldSerializer):
    """
    Open 311 vote request
    """
    node_id = serializers.CharField()
    service_code = serializers.CharField()

    class Meta:
        model = Vote
        fields= ('service_code', 'node', 'vote',)
        non_native_fields = ('node', 'service_code')


class VoteRequestSerializer(serializers.ModelSerializer):
    """
    Open 311 vote request
    """

    class Meta:
        model = Vote


class CommentRequestListSerializer(ExtraFieldSerializer):
    """
    Open 311 comment request
    """
    service_code = serializers.CharField()
    class Meta:
        model = Comment
        fields= ('service_code', 'node', 'text',)
        non_native_fields = ('node', 'service_code')


class CommentRequestSerializer(serializers.ModelSerializer):
    """
    Open 311 comment request
    """

    class Meta:
        model = Comment


class RatingRequestListSerializer(ExtraFieldSerializer):
    """
    Open 311 rating request
    """
    service_code = serializers.CharField()
    class Meta:
        model = Rating
        fields= ('service_code', 'node', 'value',)
        non_native_fields = ('node', 'service_code')


class RatingRequestSerializer(serializers.ModelSerializer):
    """
    Open 311 rating request
    """

    class Meta:
        model = Rating



from django.db import models
from django.utils.translation import ugettext_lazy as _

from nodeshot.core.nodes.models import Node
from nodeshot.core.base.choices import PLANNED_STATUS


class PlannedNode(Node):
    date = models.DateTimeField(_('planned for'))
    planning_status = models.SmallIntegerField(_('status'), max_length=1, choices=PLANNED_STATUS, default=0)
    
    class Meta:
        db_table = 'planned_node'
        permissions = (('can_view_planned_nodes', 'Can view planned nodes'),)



from django.db import models
from django.utils.translation import ugettext_lazy as _, ugettext as __
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.core.mail import send_mail
from django.core.urlresolvers import reverse, NoReverseMatch
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic

from nodeshot.core.base.models import BaseDate

from ..settings import settings, TEXTS

NOTIFICATION_TYPE_CHOICES = [(key, _(key)) for key,value in TEXTS.iteritems()]


class Notification(BaseDate):
    """
    Notification Model
    """
    type = models.CharField(_('type'), max_length=64, choices=NOTIFICATION_TYPE_CHOICES)
    from_user = models.ForeignKey(settings.AUTH_USER_MODEL,
                                  verbose_name=_('from user'),
                                  related_name='notifications_sent',
                                  blank=True, null=True)
    to_user = models.ForeignKey(settings.AUTH_USER_MODEL,
                                verbose_name=_('to user'),
                                related_name='notifications_received')

    # generic foreign key "related_object" indicates whether the
    # notification refers to any particular object
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    related_object = generic.GenericForeignKey('content_type', 'object_id')

    text = models.CharField(_('text'), max_length=120, blank=True)
    is_read = models. BooleanField(_('read?'), default=False)

    class Meta:
        app_label = 'notifications'
        ordering = ('-id',)

    def __unicode__(self):
        return 'notification #%s' % self.id

    def clean(self, *args, **kwargs):
        """ from_user and to_user must differ """
        if self.from_user and self.from_user_id == self.to_user_id:
            raise ValidationError(_('A user cannot send a notification to herself/himself'))

    def save(self, *args, **kwargs):
        """
        custom save method to send email and push notification
        """
        created = self.pk is None

        # save notification to database only if user settings allow it
        if self.check_user_settings(medium='web'):
            super(Notification, self).save(*args, **kwargs)

        if created:
            # send notifications through other mediums according to user settings
            self.send_notifications()

    def send_notifications(self):
        """ send notifications to recipient user according to her settings """
        self.send_email()

    def send_email(self):
        """ send email notification according to user settings """
        # send only if user notification setting is set to true
        if self.check_user_settings():
            send_mail(_(self.type), self.email_message, settings.DEFAULT_FROM_EMAIL, [self.to_user.email])
            return True
        else:
            # return false otherwise
            return False

    def send_mobile(self):
        """ send push notification according to user settings """
        raise NotImplementedError('mobile notifications not implemented yet')

    def check_user_settings(self, medium='email'):
        """
        Ensure user is ok with receiving this notification through the specified medium.
        Available mediums are 'web' and 'email', while 'mobile' notifications will
        hopefully be implemented in the future.
        """
        # custom notifications are always sent
        if self.type == 'custom':
            return True

        try:
            user_settings = getattr(self.to_user, '%s_notification_settings' % medium)
        except ObjectDoesNotExist:
            # user has no settings specified
            # TODO: it would be better to create the settings with default values
            return False

        user_setting_type = getattr(user_settings.__class__, self.type).user_setting_type

        if user_setting_type == 'boolean':
            return getattr(user_settings, self.type, True)
        elif user_setting_type == 'distance':
            value = getattr(user_settings, self.type, 0)
            # enabled for all related objects
            if value is 0:
                return True
            # disabled for all related objects
            elif value < 0:
                return False
            # enabled for related objects comprised in specified distance range in km
            else:
                Model = self.related_object.__class__
                geo_field = getattr(user_settings.__class__, self.type).geo_field
                geo_value = getattr(self.related_object, geo_field)
                km = value * 1000
                queryset = Model.objects.filter(**{
                    "user_id": self.to_user_id,
                    geo_field+"__distance_lte": (geo_value, km)
                })
                # if user has related object in a distance range less than or equal to
                # his prefered range (specified in number of km), return True and send the notification
                return queryset.count() >= 1

    @property
    def email_message(self):
        """ compose complete email message text """
        url = settings.SITE_URL
        hello_text = __("Hi %s," % self.to_user.get_full_name())
        action_text = __("\n\nMore details here: %s") % url
        explain_text = __(
            "This is an automatic notification sent from from %s.\n"
            "If you want to stop receiving this notification edit your"
            "email notification settings here: %s") % (settings.SITE_NAME, 'TODO')

        return "%s\n\n%s%s\n\n%s" % (hello_text, self.text, action_text, explain_text)



from django.conf.urls import patterns, url
from .settings import EMAIL_CONFIRMATION


urlpatterns = patterns('nodeshot.community.profiles.views',
    url(r'^profiles/$', 'profile_list', name='api_profile_list'),
    url(r'^profiles/(?P<username>[-.\w]+)/$', 'profile_detail', name='api_profile_detail'),
    url(r'^profiles/(?P<username>[-.\w]+)/nodes/$', 'user_nodes', name='api_user_nodes'),
    url(r'^profiles/(?P<username>[-.\w]+)/social-links/$', 'user_social_links_list', name='api_user_social_links_list'),
    url(r'^profiles/(?P<username>[-.\w]+)/social-links/(?P<pk>[0-9]+)/$', 'user_social_links_detail', name='api_user_social_links_detail'),

    url(r'^account/$', 'account_detail', name='api_account_detail'),
    url(r'^account/login/$', 'account_login', name='api_account_login'),
    url(r'^account/logout/$', 'account_logout', name='api_account_logout'),

    url(r'^account/password/$', 'account_password_change', name='api_account_password_change'),
    url(r'^account/password/reset/$', 'account_password_reset_request_key', name='api_account_password_reset_request_key'),
    url(r'^account/password/reset/(?P<uidb36>[0-9A-Za-z]+)-(?P<key>.+)/$', 'account_password_reset_from_key', name='api_account_password_reset_from_key'),
)


# email addresses
if EMAIL_CONFIRMATION:
    urlpatterns += patterns('nodeshot.community.profiles.views',
        url(r'^account/email/$', 'account_email_list', name='api_account_email_list'),
        url(r'^account/email/(?P<pk>[0-9]+)/$', 'account_email_detail', name='api_account_email_detail'),
        url(r'^account/email/(?P<pk>[0-9]+)/resend-confirmation/$', 'account_email_resend_confirmation', name='api_account_email_resend_confirmation'),
    )



from rest_framework import permissions


class IsOwnerOrReadOnly(permissions.BasePermission):
    """
    Object-level permission to only allow owners of an object to edit it.
    Assumes the model instance has an `owner` attribute.
    """

    def has_object_permission(self, request, view, obj):
        # Read permissions are allowed to any request,
        # so we'll always allow GET, HEAD or OPTIONS requests.
        if request.method in permissions.SAFE_METHODS:
            return True

        action = ''

        if request.method in ['PUT', 'PATCH']:
            action = 'change'
        elif request.method in ['DELETE']:
            action = 'delete'
        elif request.method == 'POST':
            action = 'add'

        class_name = obj.__class__.__name__
        # if node
        if class_name == 'Node':
            owner = obj.user
            layer = obj.layer
        # if image
        elif class_name == 'Image':
            owner = obj.node.user
            layer = obj.node.layer
        # if layer is external, object can't be edited
        if layer.is_external:
            return False
        # Instance must have an attribute named `owner`.
        return owner == request.user or request.user.has_perm('nodes.%s_%s' % (action, class_name.lower()))



from rest_framework import serializers

from .models import *


__all__ = [
    'PageListSerializer',
    'PageDetailSerializer',
    'MenuSerializer'
]


class PageListSerializer(serializers.ModelSerializer):
    """
    Page List Serializer
    """
    details = serializers.HyperlinkedIdentityField(view_name='api_page_detail', lookup_field='slug')

    class Meta:
        model = Page
        fields = ('title', 'slug', 'added', 'updated', 'details')


class PageDetailSerializer(PageListSerializer):
    """
    Page Detail Serializer
    """
    class Meta:
        model = Page
        fields = ('title', 'slug', 'content',
                  'meta_description', 'meta_keywords', 'meta_robots',
                  'added', 'updated', 'details')


class ChildrenSerializer(serializers.ModelSerializer):
    """
    Children Serializer
    """
    class Meta:
        model = MenuItem
        fields = ('name', 'url', 'classes')


class MenuSerializer(serializers.ModelSerializer):
    """
    Menu Serializer
    """
    children = serializers.SerializerMethodField('get_children')

    def get_children(self, obj):
        user = self.context['request'].user
        queryset = obj.menuitem_set.published().accessible_to(user)
        return ChildrenSerializer(queryset, many=True).data

    class Meta:
        model = MenuItem
        fields = ('name', 'url', 'classes', 'children')



from django.contrib.gis import admin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _

from nodeshot.core.base.admin import BaseGeoAdmin, PublishActionsAdminMixin

from .settings import REVERSION_ENABLED, TEXT_HTML
from .models import Layer

# enable django-reversion according to settings
if REVERSION_ENABLED:
    import reversion

    class GeoAdmin(BaseGeoAdmin, reversion.VersionAdmin):
        change_list_template = 'reversion_and_smuggler/change_list.html'
else:
    class GeoAdmin(BaseGeoAdmin):
        change_list_template = 'smuggler/change_list.html'


class LayerAdmin(PublishActionsAdminMixin, GeoAdmin):
    list_display = (
        'name', 'is_published', 'view_nodes',
        'organization', 'email', 'is_external',
        'new_nodes_allowed', 'added', 'updated'
    )
    list_filter = ('is_external', 'is_published')
    search_fields = ('name', 'description', 'organization', 'email')
    filter_horizontal = ('mantainers',)
    prepopulated_fields = {'slug': ('name',)}
    inlines = []

    # django-grappelli usability improvements
    raw_id_fields = ('mantainers',)
    autocomplete_lookup_fields = {
        'm2m': ['mantainers'],
    }

    if TEXT_HTML:
        # enable editor for "extended text description" only
        html_editor_fields = ['text']

    def view_nodes(self, obj):
        return '<a href="%s?layer__id__exact=%s">%s</a>' % (
            reverse('admin:nodes_node_changelist'),
            obj.pk,
            _('view nodes')
        )
    view_nodes.allow_tags = True

    def publish_action(self, request, queryset):
        super(LayerAdmin, self).publish_action(request, queryset)
        # unpublish all nodes of selected layers
        Layer.node_set.related.model.filter(layer__in=queryset).update(is_published=True)
    publish_action.short_description = _("Publish selected layers (automatically publishes all nodes of layer)")

    def unpublish_action(self, request, queryset):
        super(LayerAdmin, self).unpublish_action(request, queryset)
        # publish all nodes of selected layers
        Layer.node_set.related.model.filter(layer__in=queryset).update(is_published=False)
    unpublish_action.short_description = _("Unpublish selected layers (automatically unpublishes all nodes of layer)")


admin.site.register(Layer, LayerAdmin)



from django.db import models

from nodeshot.core.nodes.models import Node


class NodeRatingCount(models.Model):
    """
    Node Rating Count
    Keep track of participation counts of nodes.
    """
    node = models.OneToOneField(Node)
    likes = models.IntegerField(default=0)
    dislikes = models.IntegerField(default=0)
    rating_count = models.IntegerField(default=0)
    rating_avg = models.FloatField(default=0.0)
    comment_count = models.IntegerField(default=0)
    
    def __unicode__(self):
        return self.node.name
    
    class Meta:
        app_label = 'participation'
        db_table = 'participation_node_counts'


from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from rest_framework import pagination, serializers
from rest_framework.reverse import reverse
from rest_framework_gis import serializers as gis_serializers

from .models import *
from .models.choices import INTERFACE_TYPES
from .fields import MacAddressField, IPAddressField, IPNetworkField

from rest_framework_hstore.fields import HStoreField


__all__ = [
    'DeviceListSerializer',
    'DeviceDetailSerializer',
    'DeviceAddSerializer',
    'NodeDeviceListSerializer',
    'PaginatedDeviceSerializer',
    'PaginatedNodeDeviceSerializer',
    
    'EthernetSerializer',
    'EthernetDetailSerializer',
    'EthernetAddSerializer',
    
    'WirelessSerializer',
    'WirelessDetailSerializer',
    'WirelessAddSerializer',
    
    'BridgeSerializer',
    'BridgeDetailSerializer',
    'BridgeAddSerializer',
    
    'TunnelSerializer',
    'TunnelDetailSerializer',
    'TunnelAddSerializer',
    
    'VlanSerializer',
    'VlanDetailSerializer',
    'VlanAddSerializer',
    
    'VapSerializer',
    
    'IpSerializer',
    'IpDetailSerializer',
    'IpAddSerializer'
]


# ------ DEVICES ------ #

  
class DeviceListSerializer(gis_serializers.GeoModelSerializer):
    """ location geo serializer  """
    
    node = serializers.Field(source='node.slug')
    type = serializers.WritableField(source='get_type_display', label=_('type'))
    status = serializers.Field(source='get_status_display')
    details = serializers.HyperlinkedIdentityField(view_name='api_device_details')
    
    class Meta:
        model = Device
        fields = [
            'id', 'node', 'name', 'type', 'status',
            'location', 'elev',
            'os', 'os_version', 'description',
            'first_seen', 'last_seen',
            'added', 'updated', 'details'
        ]
        read_only_fields = [
            'first_seen', 'last_seen',
            'added', 'updated'
        ]


class DeviceDetailSerializer(DeviceListSerializer):
    
    access_level = serializers.Field(source='get_access_level_display')
    
    routing_protocols_named = serializers.RelatedField(source='routing_protocols', many=True)
    
    ethernet = serializers.SerializerMethodField('get_ethernet_interfaces')
    ethernet_url = serializers.HyperlinkedIdentityField(view_name='api_device_ethernet')
    
    wireless = serializers.SerializerMethodField('get_wireless_interfaces')
    wireless_url = serializers.HyperlinkedIdentityField(view_name='api_device_wireless')
    
    bridge = serializers.SerializerMethodField('get_bridge_interfaces')
    bridge_url = serializers.HyperlinkedIdentityField(view_name='api_device_bridge')
    
    tunnel = serializers.SerializerMethodField('get_tunnel_interfaces')
    tunnel_url = serializers.HyperlinkedIdentityField(view_name='api_device_tunnel')
    
    vlan = serializers.SerializerMethodField('get_vlan_interfaces')
    vlan_url = serializers.HyperlinkedIdentityField(view_name='api_device_vlan')
    
    data = HStoreField(
        required=False,
        label=_('extra data'),
        help_text=_('store extra attributes in JSON string')
    )
    
    def get_ethernet_interfaces(self, obj):
        user = self.context['request'].user
        interfaces = Ethernet.objects.filter(device=obj.id).accessible_to(user)
        return EthernetSerializer(interfaces, many=True, context=self.context).data
    
    def get_wireless_interfaces(self, obj):
        user = self.context['request'].user
        interfaces = Wireless.objects.filter(device=obj.id).accessible_to(user)
        return WirelessSerializer(interfaces, many=True, context=self.context).data
    
    def get_bridge_interfaces(self, obj):
        user = self.context['request'].user
        interfaces = Bridge.objects.filter(device=obj.id).accessible_to(user)
        return BridgeSerializer(interfaces, many=True, context=self.context).data
    
    def get_tunnel_interfaces(self, obj):
        user = self.context['request'].user
        interfaces = Tunnel.objects.filter(device=obj.id).accessible_to(user)
        return TunnelSerializer(interfaces, many=True, context=self.context).data
    
    def get_vlan_interfaces(self, obj):
        user = self.context['request'].user
        interfaces = Vlan.objects.filter(device=obj.id).accessible_to(user)
        return VlanSerializer(interfaces, many=True, context=self.context).data
    
    class Meta:
        model = Device
        primary_fields = [
            'id', 'access_level', 'node', 'name', 'type', 'status',
            'location', 'elev',
            'os', 'os_version', 'description',
            'routing_protocols', 'routing_protocols_named',
            'first_seen', 'last_seen', 'data',
            'added', 'updated'
        ]
        
        secondary_fields = [
            'ethernet', 'ethernet_url',
            'wireless', 'wireless_url',
            'bridge', 'bridge_url',
            'tunnel', 'tunnel_url',
            'vlan', 'vlan_url'
        ]
        
        fields = primary_fields + secondary_fields
        read_only_fields = [
            'added', 'updated',
            'first_seen', 'last_seen',
        ]


class NodeDeviceListSerializer(DeviceDetailSerializer):
    """ serializer to list devices of a node """
    class Meta:
        model = Device
        fields = DeviceDetailSerializer.Meta.primary_fields[:] + ['details']


class DeviceAddSerializer(NodeDeviceListSerializer):
    """ Serializer for Device Creation """
    node = serializers.WritableField(source='node_id')
    type = serializers.WritableField(source='type')
    details = serializers.HyperlinkedIdentityField(view_name='api_device_details') 


class PaginatedDeviceSerializer(pagination.PaginationSerializer):
    class Meta:
        object_serializer_class = DeviceListSerializer


class PaginatedNodeDeviceSerializer(pagination.PaginationSerializer):
    class Meta:
        object_serializer_class = NodeDeviceListSerializer


# ------ INTERFACES ------ #


class InterfaceSerializer(serializers.ModelSerializer):
    """ Base Interface Serializer Class """
    access_level = serializers.Field(source='get_access_level_display')
    mac = MacAddressField(label=_('mac address'))
    type = serializers.Field(source='get_type_display', label=_('type'))
    
    tx_rate = serializers.Field()
    rx_rate = serializers.Field()
    
    ip = serializers.SerializerMethodField('get_ip_addresses')
    ip_url = serializers.HyperlinkedIdentityField(view_name='api_interface_ip')
    
    data = HStoreField(
        required=False,
        label=_('extra data'),
        help_text=_('store extra attributes in JSON string')
    )
    
    def get_ip_addresses(self, obj):
        user = self.context['request'].user
        interfaces = Ip.objects.filter(interface=obj.id).accessible_to(user)
        return IpSerializer(interfaces, many=True, context=self.context).data
    
    class Meta:
        model = Interface
        fields = [
            'id', 'access_level', 'type', 'name',
            'mac', 'mtu', 'tx_rate', 'rx_rate',
            'data', 'added', 'updated', 'ip_url', 'ip',
        ]
        read_only_fields = ['added', 'updated']


class EthernetSerializer(InterfaceSerializer):
    class Meta:
        model = Ethernet
        fields = InterfaceSerializer.Meta.fields[:] + [
            'standard', 'duplex'
        ]


class EthernetDetailSerializer(EthernetSerializer):
    details = serializers.HyperlinkedIdentityField(view_name='api_ethernet_details')
    
    class Meta:
        model = Ethernet
        fields = EthernetSerializer.Meta.fields[:] + ['details']
        read_only_fields = ('added', 'updated')


class EthernetAddSerializer(EthernetSerializer):
    class Meta:
        model = Ethernet
        fields = EthernetSerializer.Meta.fields[:] + ['device']
        read_only_fields = ('added', 'updated')


class VapSerializer(serializers.ModelSerializer):
    class Meta:
        model = Vap
        fields = ['essid', 'bssid', 'encryption']


class WirelessSerializer(InterfaceSerializer):
    vap = VapSerializer(source='vap_set', many=True, read_only=True)
    
    class Meta:
        model = Wireless
        fields = InterfaceSerializer.Meta.fields[:] + [
            'mode', 'standard', 'channel',
            'channel_width', 'output_power', 'dbm', 'noise', 'vap'
        ]


class WirelessDetailSerializer(WirelessSerializer):
    details = serializers.HyperlinkedIdentityField(view_name='api_wireless_details')
    
    class Meta:
        model = Wireless
        fields = WirelessSerializer.Meta.fields[:] + ['details']
        read_only_fields = ('added', 'updated')


class WirelessAddSerializer(WirelessSerializer):
    class Meta:
        model = Wireless
        fields = WirelessSerializer.Meta.fields[:] + ['device']
        read_only_fields = ('added', 'updated')


from .models.interfaces.bridge import validate_bridged_interfaces

class BridgeSerializer(InterfaceSerializer):
    
    interfaces = serializers.PrimaryKeyRelatedField(many=True, required=True)
    interfaces_links = serializers.SerializerMethodField('get_interfaces_links')
    
    def get_interfaces_links(self, obj):
        user = self.context['request'].user
        links = []
        
        for interface in obj.interfaces.accessible_to(user):
            view_names = {
                INTERFACE_TYPES['ethernet']: 'api_ethernet_details',
                INTERFACE_TYPES['wireless']: 'api_wireless_details',
                INTERFACE_TYPES['bridge']: 'api_bridge_details',
                INTERFACE_TYPES['virtual']: 'api_tunnel_details',
                INTERFACE_TYPES['vlan']: 'api_vlan_details'
            }
            link = reverse(view_names[interface.type],
                           args=[interface.id],
                           request=self.context['request'],
                           format=self.context['format'])
            links.append(link)
        
        return links
    
    def validate(self, attrs):
        """ perform many2many validation """
        interfaces = attrs.get('interfaces', [])
        created = False
        
        # redundant but necessary
        if(len(interfaces) < 2 and not self.partial) or \
          (self.partial and 'interfaces' in attrs and len(interfaces) < 2):  # this line adds support for partial udpates with PATCH method
            raise ValidationError(_(u'You must bridge at least 2 interfaces'))
        
        # when creating a new interface self.object is None
        if not self.object:
            created = True
            self.object = Bridge(
                device_id=attrs['device'].id,
                name=attrs['name'],
                mac=attrs['mac']
            )
            self.object.full_clean()
            self.object.save()
        
        # validate many2many
        try:
            validate_bridged_interfaces(
                sender=self.object.interfaces,
                instance=self.object,
                action="pre_add",
                reverse=False,
                model=self.object.interfaces.model,
                pk_set=[interface.id for interface in interfaces]
            )
        except ValidationError as e:
            # delete object first if validation error raised and then raise again
            if created:
                self.object.delete()
            raise ValidationError(e.messages[0])
        
        # delete any created object that was needed for validation only
        if created:
            self.object.delete()
        
        return attrs
    
    class Meta:
        model = Bridge
        fields = InterfaceSerializer.Meta.fields[:] + ['interfaces', 'interfaces_links']
        read_only_fields = ('added', 'updated')


class BridgeDetailSerializer(BridgeSerializer):
    details = serializers.HyperlinkedIdentityField(view_name='api_bridge_details')
    
    class Meta:
        model = Bridge
        fields = BridgeSerializer.Meta.fields[:] + ['details']
        read_only_fields = ('added', 'updated')


class BridgeAddSerializer(BridgeSerializer):
    class Meta:
        model = Bridge
        fields = BridgeSerializer.Meta.fields[:] + ['device', 'interfaces']
        read_only_fields = ('added', 'updated')


class TunnelSerializer(InterfaceSerializer):
    class Meta:
        model = Tunnel
        fields = InterfaceSerializer.Meta.fields[:] + [
            'sap', 'protocol', 'endpoint',
        ]


class TunnelDetailSerializer(TunnelSerializer):
    details = serializers.HyperlinkedIdentityField(view_name='api_tunnel_details')
    
    class Meta:
        model = Tunnel
        fields = TunnelSerializer.Meta.fields[:] + ['details']
        read_only_fields = ('added', 'updated')


class TunnelAddSerializer(TunnelSerializer):
    class Meta:
        model = Tunnel
        fields = TunnelSerializer.Meta.fields[:] + ['device']
        read_only_fields = ('added', 'updated')


class VlanSerializer(InterfaceSerializer):
    class Meta:
        model = Vlan
        fields = InterfaceSerializer.Meta.fields[:] + ['tag']


class VlanDetailSerializer(VlanSerializer):
    details = serializers.HyperlinkedIdentityField(view_name='api_vlan_details')
    
    class Meta:
        model = Vlan
        fields = VlanSerializer.Meta.fields[:] + ['details']
        read_only_fields = ('added', 'updated')


class VlanAddSerializer(VlanSerializer):
    class Meta:
        model = Vlan
        fields = VlanSerializer.Meta.fields[:] + ['device']
        read_only_fields = ('added', 'updated')


# ------ IP ADDRESS ------ #


class IpSerializer(serializers.ModelSerializer):
    address = IPAddressField()
    netmask = IPNetworkField()
    
    class Meta:
        model = Ip
        fields = ['address', 'protocol', 'netmask']


class IpAddSerializer(IpSerializer):
    class Meta:
        model = Ip
        fields = IpSerializer.Meta.fields[:] + ['interface']
        read_only_fields = ('added', 'updated')


class IpDetailSerializer(IpSerializer):
    details = serializers.HyperlinkedIdentityField(view_name='api_ip_details')
    
    class Meta:
        model = Ip
        fields = ['id'] + IpSerializer.Meta.fields[:] + ['added', 'updated', 'details']
        read_only_fields = ('added', 'updated')



from django.conf.urls import patterns, url


urlpatterns = patterns('nodeshot.interop.open311.views',
    url(r'^open311/$', 'service_discovery', name='api_service_discovery'),
    url(r'^open311/services.json$', 'service_definition_list', name='api_service_definition_list'),
    url(r'^open311/services/(?P<service_type>[-\w]+).json$', 'service_definition_detail', name='api_service_definition_detail'),
    url(r'^open311/requests.json$', 'service_request_list', name='api_service_request_list'),
    url(r'^open311/requests/(?P<service_code>[-\w]+)-(?P<request_id>[-\w]+).json$', 'service_request_detail', name='api_service_request_detail'),
)



import sys

from cStringIO import StringIO

from django.test import TestCase
from django.core import management
from django.core.urlresolvers import reverse
from django.core.exceptions import ValidationError
from django.contrib.gis.geos import Point, GEOSGeometry

from nodeshot.core.layers.models import Layer
from nodeshot.core.nodes.models import Node
from nodeshot.core.base.tests import user_fixtures

from .models import LayerExternal, NodeExternal
from .settings import settings, SYNCHRONIZERS
from .tasks import synchronize_external_layers


TEST_FILES_PATH = '%snodeshot/testing' % settings.STATIC_URL


def capture_output(command, args=[], kwargs={}):
    """ captures standard output """
    # start capturing output
    output = StringIO()
    sys.stdout = output
    # execute command
    command(*args, **kwargs)
    # stop capturing print statements
    sys.stdout = sys.__stdout__
    # return captured output
    return output.getvalue()


class SyncTest(TestCase):
    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json'
    ]

    def test_external_layer_representation(self):
        self.assertIn('object', str(LayerExternal()))
        e = LayerExternal()
        e.layer = Layer.objects.external().first()
        self.assertIn('config', str(e))

    def test_external_layer_creation(self):
        layer = Layer()
        layer.name = 'test'
        layer.slug = 'test'
        layer.description = 'test'
        layer.is_external = True
        layer.organization = 'test'
        layer.area = Point(8.96166, 44.4185)
        layer.full_clean()
        layer.save()

    def test_node_external_creation(self):
        n = NodeExternal()
        n.node = Node.objects.first()
        n.external_id = 7
        n.full_clean()
        n.save()

    def test_not_interoperable(self):
        """ test not interoperable """
        output = capture_output(management.call_command, ('sync', 'vienna'))
        # ensure following text is in output
        self.assertIn('does not have a synchronizer class specified', output)

    def test_reload_schema_config(self):
        external = LayerExternal(layer=Layer.objects.external().first())
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.Nodeshot'
        external._reload_schema()
        self.assertTrue(external.config.field.schema_mode)
        self.assertFalse(external.config.field.editable)
        external.synchronizer_path = 'None'
        external._reload_schema()
        self.assertFalse(external.config.field.schema_mode)
        self.assertFalse(external.config.field.editable)

    def test_management_command_exclude(self):
        """ test --exclude """
        output = capture_output(
            management.call_command,
            ['sync'],
            {'exclude': 'vienna,test'}
        )
        self.assertIn('no layers to process', output)

    def test_celery_task(self):
        """ ensure celery task works as expected """
        output = capture_output(synchronize_external_layers.apply)
        self.assertIn('does not have a synchronizer class specified', output)

    def test_celery_task_with_arg(self):
        output = capture_output(synchronize_external_layers.apply, [['vienna']])
        self.assertIn('does not have a synchronizer class specified', output)

    def test_celery_task_with_exclude(self):
        output = capture_output(
            synchronize_external_layers.apply,
            kwargs={'kwargs': {'exclude': 'vienna,test'}}
        )
        self.assertIn('no layers to process', output)

    def test_celery_task_with_error(self):
        try:
            synchronize_external_layers.apply(['wrongvalue'])
            self.fail('should have got exception')
        except management.CommandError as e:
            self.assertIn('wrongvalue', str(e))
            self.assertIn('does not exist', str(e))

    def test_layer_admin(self):
        """ ensure layer admin does not return any error """
        layer = Layer.objects.external()[0]
        url = reverse('admin:layers_layer_change', args=[layer.id])
        self.client.login(username='admin', password='tester')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.Nodeshot'
        external._reload_schema()
        external.layer_url = "http://test.com/"
        external.full_clean()
        external.save()

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        layer = Layer.objects.filter(is_external=False)[0]
        url = reverse('admin:layers_layer_change', args=[layer.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        url = reverse('admin:layers_layer_add')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_config_validation(self):
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)
        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.Nodeshot'
        external.full_clean()  # no error

        # after reloading the schema we expect to get a validation error
        external._reload_schema()
        with self.assertRaises(ValidationError):
            external.full_clean()

    def test_layer_external_reload_schema_view(self):
        external = LayerExternal(layer=Layer.objects.external().first())
        external.full_clean()
        external.save()
        # login
        self.client.login(username='admin', password='tester')
        url = reverse('layer_external_reload_schema', args=[external.layer_id])
        # loop over all synchronizer and try them all
        for sync_tuple in SYNCHRONIZERS:
            path = sync_tuple[0]
            response = self.client.post(url, {"synchronizer_path": path})
            # ensure http response is ok
            self.assertEqual(response.status_code, 200)
            # ensure data has really changed
            external = LayerExternal.objects.get(pk=external.pk)
            self.assertEqual(external.synchronizer_path, path)

    def test_admin_synchronize_action(self):
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        url = '%s/geojson1.json' % TEST_FILES_PATH

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.GeoJson'
        external._reload_schema()
        external.url = url
        external.full_clean()
        external.save()

        from django.http.request import HttpRequest
        from django.contrib.admin.sites import AdminSite
        from nodeshot.interop.sync.admin import LayerAdmin

        admin = LayerAdmin(Layer, AdminSite())
        request = HttpRequest()

        # expect no output because trying to synchronize non-external layers
        output = capture_output(
            admin.synchronize_action,
            [request, Layer.objects.filter(is_external=False)]
        )

        self.assertEqual(output, '')

        # expect no output because trying to synchronize a single non-external layer
        output = capture_output(
            admin.synchronize_action,
            [request, Layer.objects.filter(is_external=False)[0:1]]
        )

        self.assertEqual(output, '')

        # expects output
        output = capture_output(
            admin.synchronize_action,
            [request, Layer.objects.filter(pk=layer.pk)]
        )

        # ensure following text is in output
        self.assertIn('2 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

        # ensure all nodes have been imported
        self.assertEqual(layer.node_set.count(), 2)

    def test_openwisp(self):
        """ test OpenWisp synchronizer """
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.OpenWisp'
        external._reload_schema()
        external.url = '%s/openwisp-georss.xml' % TEST_FILES_PATH
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('42 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('42 total external', output)
        self.assertIn('42 total local', output)

        # start checking DB too
        nodes = layer.node_set.all()

        # ensure all nodes have been imported
        self.assertEqual(nodes.count(), 42)

        # check one particular node has the data we expect it to have
        node = Node.objects.get(slug='podesta1-ced')
        self.assertEqual(node.name, 'Podesta1 CED')
        self.assertEqual(node.address, 'Test WISP')
        point = Point(8.96166, 44.4185)
        self.assertTrue(node.geometry.equals(point))
        self.assertEqual(node.updated.strftime('%Y-%m-%d'), '2013-07-10')
        self.assertEqual(node.added.strftime('%Y-%m-%d'), '2011-08-24')

        # --- with the following step we expect some nodes to be deleted --- #

        external.url = '%s/openwisp-georss2.xml' % TEST_FILES_PATH
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('4 nodes unmodified', output)
        self.assertIn('38 nodes deleted', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('4 total external', output)
        self.assertIn('4 total local', output)

        # ensure all nodes have been imported
        self.assertEqual(nodes.count(), 4)

        # check one particular node has the data we expect it to have
        node = Node.objects.get(slug='lercari2-42')
        self.assertEqual(node.name, 'Lercari2 42')
        self.assertEqual(node.address, 'Test WISP')
        point = Point(8.96147, 44.4076)
        self.assertTrue(node.geometry.equals(point))
        self.assertEqual(node.updated.strftime('%Y-%m-%d'), '2013-07-10')
        self.assertEqual(node.added.strftime('%Y-%m-%d'), '2013-06-14')

    def test_geojson_sync(self):
        """ test GeoJSON sync """
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.GeoJson'
        external._reload_schema()
        external.url = '%s/geojson1.json' % TEST_FILES_PATH
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('2 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

        # ensure all nodes have been imported
        self.assertEqual(layer.node_set.count(), 2)

        # check one particular node has the data we expect it to have
        node = Node.objects.get(slug='simplegeojson')
        self.assertEqual(node.name, 'simplegeojson')
        self.assertIn('simplegeojson', node.address)
        geometry = GEOSGeometry('{"type":"Polygon","coordinates":[[[12.501493164066,41.990441051094],[12.583890625003,41.957770034531],[12.618222900394,41.912820024702],[12.607923217778,41.877552973685],[12.582088180546,41.82423212474],[12.574148841861,41.813357913568],[12.551532455447,41.799730560554],[12.525053688052,41.795155470656],[12.510505386356,41.793715689492],[12.43308610535,41.803249638226],[12.388883300784,41.813613798573],[12.371030517581,41.870906276755],[12.382016845706,41.898511105474],[12.386136718753,41.912820024702],[12.38064355469,41.926104006681],[12.38064355469,41.955727539561],[12.413602539065,41.974107637675],[12.445188232426,41.983295698272],[12.45617456055,41.981254021593],[12.476773925785,41.985337309484],[12.490506835941,41.985337309484],[12.506986328129,41.990441051094],[12.501493164066,41.990441051094]]]}')
        self.assertTrue(node.geometry.equals_exact(geometry) or node.geometry.equals(geometry))
        self.assertEqual(node.elev, 10.0)

        # --- repeat --- #

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('2 nodes unmodified', output)
        self.assertIn('0 nodes deleted', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

        # --- repeat with slightly different input --- #

        external.url = '%s/geojson2.json' % TEST_FILES_PATH
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('1 nodes unmodified', output)
        self.assertIn('0 nodes deleted', output)
        self.assertIn('1 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

    def test_preexisting_name(self):
        """ test preexisting names """
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        node = Node.first()
        self.assertNotEqual(layer.id, node.layer.id)
        node.name = 'simplejson'
        node.save()

        url = '%s/geojson1.json' % TEST_FILES_PATH

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.GeoJson'
        external._reload_schema()
        external.config = {"url": url}
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('2 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

    def test_key_mappings(self):
        """ importing a file with different keys """
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        node = Node.first()
        self.assertNotEqual(layer.id, node.layer.id)
        node.name = 'simplejson'
        node.save()

        url = '%s/geojson3.json' % TEST_FILES_PATH

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.GeoJson'
        external._reload_schema()
        external.url = url
        external.field_name = "nome"
        external.field_description = "descrizione"
        external.field_address = "indirizzo"
        external.field_elev = "altitudine"
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('2 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

        node = Node.objects.get(slug='verycool')
        self.assertEqual(node.name, 'veryCool')
        self.assertEqual(node.address, 'veryCool indirizzo')
        self.assertEqual(node.description, 'veryCool descrizione')
        self.assertEqual(node.elev, 10.0)

        node = Node.objects.get(slug='secondo')
        self.assertEqual(node.name, 'secondo')
        self.assertEqual(node.address, 'secondo indirizzo')
        self.assertEqual(node.description, 'secondo descrizione')
        self.assertEqual(node.elev, 20.0)

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )
        # no changes
        self.assertIn('0 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('0 nodes deleted', output)
        self.assertIn('2 nodes unmodified', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

    def test_georss_simple(self):
        """ test GeoRSS simple """
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        url = '%s/georss-simple.xml' % TEST_FILES_PATH

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.GeoRss'
        external._reload_schema()
        external.config = {"url": url}
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('3 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('3 total external', output)
        self.assertIn('3 total local', output)

        # start checking DB too
        nodes = layer.node_set.all()

        # ensure all nodes have been imported
        self.assertEqual(nodes.count(), 3)

        # check one particular node has the data we expect it to have
        node = Node.objects.get(slug='item-2')
        self.assertEqual(node.name, 'item 2')
        self.assertEqual(node.address, '')
        self.assertEqual(node.updated.strftime('%Y-%m-%d'), '2006-08-17')
        geometry = GEOSGeometry('POINT (-70.92 44.256)')
        self.assertTrue(node.geometry.equals_exact(geometry) or node.geometry.equals(geometry))

        # --- repeat --- #

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('3 nodes unmodified', output)
        self.assertIn('0 nodes deleted', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('3 total external', output)
        self.assertIn('3 total local', output)

    def test_georss_w3c(self):
        """ test GeoRSS w3c """
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = False
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        url = '%s/georss-w3c.xml' % TEST_FILES_PATH

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.GeoRss'
        external._reload_schema()
        external.config = {"url": url}
        external.full_clean()
        external.save()

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('2 nodes added', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

        # start checking DB too
        nodes = layer.node_set.all()

        # ensure all nodes have been imported
        self.assertEqual(nodes.count(), 2)

        # check one particular node has the data we expect it to have
        node = Node.objects.get(slug='test-2')
        self.assertEqual(node.name, 'test 2')
        self.assertEqual(node.address, '')
        geometry = GEOSGeometry('POINT (95.8932 5.6319)')
        self.assertTrue(node.geometry.equals_exact(geometry) or node.geometry.equals(geometry))

        # --- repeat --- #

        output = capture_output(
            management.call_command,
            ['sync', 'vienna'],
            kwargs={'verbosity': 0}
        )

        # ensure following text is in output
        self.assertIn('2 nodes unmodified', output)
        self.assertIn('0 nodes deleted', output)
        self.assertIn('0 nodes changed', output)
        self.assertIn('2 total external', output)
        self.assertIn('2 total local', output)

    def test_nodeshot_sync(self):
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = True
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        external = LayerExternal(layer=layer)
        external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.Nodeshot'
        external._reload_schema()
        external.layer_url = "%s/api/v1/layers/rome/" % settings.SITE_URL
        external.full_clean()
        external.verify_ssl = False
        external.full_clean()
        external.save()

        # standard node list
        url = reverse('api_layer_nodes_list', args=[layer.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('nodes', response.data)
        self.assertNotIn('name', response.data)
        self.assertNotIn('description', response.data)
        self.assertEqual(type(response.data['results']), list)
        self.assertEqual(type(response.data['results'][0]), dict)

        # limit pagination to 1
        url = reverse('api_layer_nodes_list', args=[layer.slug])
        response = self.client.get('%s?limit=1&page=2' % url)
        self.assertEqual(len(response.data['results']), 1)
        self.assertIn(settings.SITE_URL, response.data['previous'])
        self.assertIn(url, response.data['previous'])
        self.assertIn(settings.SITE_URL, response.data['next'])
        self.assertIn(url, response.data['next'])

        # geojson view
        url = reverse('api_layer_nodes_geojson', args=[layer.slug])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data['type'], 'FeatureCollection')
        self.assertEqual(type(response.data['features']), list)
        self.assertEqual(type(response.data['features'][0]), dict)

        # limit pagination to 1 in geojson view
        url = reverse('api_layer_nodes_geojson', args=[layer.slug])
        response = self.client.get('%s?limit=1&page=2' % url)
        self.assertEqual(len(response.data['features']), 1)

    def test_nodeshot_sync_exceptions(self):
        layer = Layer.objects.external()[0]
        layer.new_nodes_allowed = True
        layer.save()
        layer = Layer.objects.get(pk=layer.pk)

        external = LayerExternal(layer=layer)

        for layer_url in [
            'http://idonotexi.st.com/hey',
            'https://test.map.ninux.org/api/v1/layers/'
        ]:
            external.synchronizer_path = 'nodeshot.interop.sync.synchronizers.Nodeshot'
            external._reload_schema()
            external.layer_url = layer_url
            external.full_clean()
            external.verify_ssl = False
            external.full_clean()
            external.save()

            url = reverse('api_layer_nodes_list', args=[layer.slug])
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertIn('error', response.data)
            self.assertIn('exception', response.data)
            # geojson view
            url = reverse('api_layer_nodes_geojson', args=[layer.slug])
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertIn('error', response.data)
            self.assertIn('exception', response.data)



from datetime import datetime
from threading import Thread
from influxdb import client

from . import settings


def get_db():
    """Returns an ``InfluxDBClient`` instance."""
    return client.InfluxDBClient(
        settings.INFLUXDB_HOST,
        settings.INFLUXDB_PORT,
        settings.INFLUXDB_USER,
        settings.INFLUXDB_PASSWORD,
        settings.INFLUXDB_DATABASE,
    )


def query(query, params={}, expected_response_code=200,
          database=None, raw=False):
    """Wrapper around ``InfluxDBClient.query()``."""
    db = get_db()
    return db.query(query, params,
                    expected_response_code,
                    database=database or settings.INFLUXDB_DATABASE,
                    raw=raw)


def write(name, values, tags={}, timestamp=None, database=None):
    """ write metrics """
    thread = Thread(target=write_threaded,
                    args=(name, values, tags, timestamp, database))
    thread.start()


def write_threaded(name, values, tags={}, timestamp=None, database=None):
    """ Method to be called via threading module. """
    point = {
        'name': name,
        'tags': tags,
        'fields': values
    }
    if isinstance(timestamp, datetime):
        timestamp = timestamp.strftime('%Y-%m-%dT%H:%M:%SZ')
    if timestamp:
        point['timestamp'] = timestamp
    try:
        get_db().write({
            'database': database or settings.INFLUXDB_DATABASE,
            'points': [point]
        })
    except Exception, e:
        if settings.INFLUXDB_FAIL_SILENTLY:
            pass
        else:
            raise e


def create_database():
    """ creates database if necessary """
    db = get_db()
    response = db.query('SHOW DATABASES', raw=True)
    first_line = response['results'][0]['series'][0]
    try:
        databases = [database[0] for database in first_line['values']]
    except KeyError:
        databases = []
    # if database does not exists, create it
    if settings.INFLUXDB_DATABASE not in databases:
        db.create_database(settings.INFLUXDB_DATABASE)
        print('Created inlfuxdb database {0}'.format(settings.INFLUXDB_DATABASE))



import os


# ------ BEGIN DON'T TOUCH AREA ------ #

SITE_ROOT = os.path.dirname(os.path.realpath(__file__))

SECRET_KEY = '{{ secret_key }}'

ROOT_URLCONF = '{{ project_name }}.urls' #

WSGI_APPLICATION = '{{ project_name }}.wsgi.application'

# ------ END DON'T TOUCH AREA ------ #


DEBUG = True
DOMAIN = '<domain>'
SUBDIR = '/'  # edit this line if you are installing in a sub directory, like /nodeshot
SITE_NAME = '{{ project_name }}'  # site name, you can change this

DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': 'nodeshot',
        'USER': '<user>',
        'PASSWORD': '<password>',
        'HOST': '127.0.0.1',
        'PORT': '',
    },
    # Import data from older versions
    # More info about this feature here: http://nodeshot.readthedocs.org/en/latest/topics/oldimporter.html
    #'old_nodeshot': {
    #    'ENGINE': 'django.db.backends.mysql',
    #    'NAME': 'nodeshot',
    #    'USER': 'user',
    #    'PASSWORD': 'password',
    #    'OPTIONS': {
    #           "init_command": "SET storage_engine=INNODB",
    #    },
    #    'HOST': '',
    #    'PORT': '',
    #}
}

# sentry integration
#RAVEN_CONFIG = {
#    'dsn': 'https://<api-public-key>:<api-secret-key>@<sentry.host>/<id>?timeout=5&verify_ssl=0',
#}

# import the default nodeshot settings
# do not move this import
from nodeshot.conf.settings import *


# ------ All settings customizations must go here ------ #


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# In a Windows environment this must be set to your system time zone.
TIME_ZONE = 'Europe/Amsterdam'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-gb'

ADMINS = (
    #('Your name', 'your@email.com'),
)

MANAGERS = ADMINS

EMAIL_HOST = 'localhost'
EMAIL_HOST_USER = 'root@localhost'
DEFAULT_FROM_EMAIL = EMAIL_HOST_USER
SERVER_EMAIL = EMAIL_HOST_USER  # used for error reporting

LEAFLET_CONFIG.update({
    'DEFAULT_CENTER': (49.06775, 30.62011),
    'DEFAULT_ZOOM': 4,
    'MIN_ZOOM': 1,
    'MAX_ZOOM': 18,
    # Uncomment to customize map tiles
    # More information here: https://github.com/makinacorpus/django-leaflet#default-tiles-layer
    #'TILES': [
    #    ('Map', 'http://otile1.mqcdn.com/tiles/1.0.0/map/{z}/{x}/{y}.png', '&copy; <a href="http://www.openstreetmap.org/copyright" target="_blank">OpenStreetMap</a> contributors | Tiles Courtesy of <a href="http://www.mapquest.com/" target="_blank">MapQuest</a> &nbsp;<img src="http://developer.mapquest.com/content/osm/mq_logo.png">'),
    #    ('Satellite', 'http://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}', 'Source: <a href="http://www.esri.com/">Esri</a> &copy; and the GIS User Community ')
    #],
})

# social auth
FACEBOOK_APP_ID = ''
FACEBOOK_API_SECRET = ''
GOOGLE_OAUTH2_CLIENT_ID = ''
GOOGLE_OAUTH2_CLIENT_SECRET = ''
GITHUB_APP_ID = ''
GITHUB_API_SECRET = ''

# Uncomment this section to enable periodic tasks
#
#from datetime import timedelta
#
#CELERYBEAT_SCHEDULE.update({
#    'synchronize': {
#        'task': 'nodeshot.interop.sync.tasks.synchronize_external_layers',
#        'schedule': timedelta(hours=12),
#    },
#    # example of how to synchronize one of the layers with a different periodicity
#    'synchronize': {
#        'task': 'nodeshot.interop.sync.tasks.synchronize_external_layers',
#        'schedule': timedelta(minutes=30),
#        'args': ('layer_slug',)
#    },
#    # example of how to synchronize all layers except two layers
#    'synchronize': {
#        'task': 'nodeshot.interop.sync.tasks.synchronize_external_layers',
#        'schedule': timedelta(hours=12),
#        'kwargs': { 'exclude': 'layer1-slug,layer2-slug' }
#    }
#})



"""
Unit tests for participation app
"""
from django.contrib.auth import get_user_model
User = get_user_model()
from django.test import TestCase
from django.core.urlresolvers import reverse

import simplejson as json

from nodeshot.core.nodes.models import Node
from nodeshot.core.layers.models import Layer
from nodeshot.core.base.tests import user_fixtures

from .models import Comment, Rating, Vote


class ParticipationTests(TestCase):
    """ Models tests """

    fixtures = [
        'initial_data.json',
        user_fixtures,
        'test_layers.json',
        'test_status.json',
        'test_nodes.json',
        'test_images.json'
    ]

    def test_added_methods(self):
        node = Node.objects.get(pk=1)
        # ensure noderatingcount related object is automatically created with signals
        node.rating_count
        node.participation_settings
        node.layer.participation_settings

    def test_update_comment_count(self):
        """
        Comment count should be updated when a comment is created or deleted
        """
        # ensure comment count is 0
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.comment_count)
        # create a new comment
        comment = Comment(node_id=1, user_id=1, text='test comment')
        comment.save()
        # now should have incremented by 1
        node = Node.objects.get(pk=1)
        self.assertEqual(1, node.rating_count.comment_count)

        # delete comment and ensure the count gets back to 0
        comment.delete()
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.comment_count)

    def test_update_rating_count(self):
        """
        Rating count and average should be updated when a rating is created or deleted
        """
        # ensure rating count is 0
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.rating_count)
        self.assertEqual(0, node.rating_count.rating_avg)
        # create a new rating
        rating = Rating(node_id=1, user_id=1, value=10)
        rating.save()
        # now should have incremented by 1 and average should be 10
        node = Node.objects.get(pk=1)
        self.assertEqual(1, node.rating_count.rating_count)
        self.assertEqual(10, node.rating_count.rating_avg)

        # delete rating and ensure the counts get back to 0
        rating.delete()
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.rating_count)
        self.assertEqual(0, node.rating_count.rating_avg)

    def test_update_vote_count(self):
        """
        Likes and dislikes count should be updated when a vote is created or deleted
        """
        # ensure likes and dislikes count are 0
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.likes)
        self.assertEqual(0, node.rating_count.dislikes)
        # create a new like
        like = Vote(node_id=1, user_id=1, vote=1)
        like.save()
        # likes should have incremented by 1
        node = Node.objects.get(pk=1)
        self.assertEqual(1, node.rating_count.likes)
        self.assertEqual(0, node.rating_count.dislikes)
        # create a new dislike
        dislike = Vote(node_id=1, user_id=2, vote=-1)
        dislike.save()
        # dislikes should have incremented by 1
        node = Node.objects.get(pk=1)
        self.assertEqual(1, node.rating_count.likes)
        self.assertEqual(1, node.rating_count.dislikes)

        # delete like and count should get back to 0
        like.delete()
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.likes)
        self.assertEqual(1, node.rating_count.dislikes)

        # delete dislike and count should get back to 0
        dislike.delete()
        node = Node.objects.get(pk=1)
        self.assertEqual(0, node.rating_count.likes)
        self.assertEqual(0, node.rating_count.dislikes)

    def test_rating_model(self):
        self.assertEqual(Rating.objects.count(), 0)
        user = User.objects.first()
        node = Node.objects.first()
        r = Rating(user=user, node=node, value=5)
        r.full_clean()
        r.save()
        self.assertEqual(Rating.objects.count(), 1)
        self.assertEqual(Rating.objects.last().value, 5)

        r = Rating(user=user, node=node, value=2)
        r.full_clean()
        r.save()
        self.assertEqual(Rating.objects.count(), 1)
        self.assertEqual(Rating.objects.last().value, 2)

    def test_node_comment_api(self):
        """
        Comments endpoint should be reachable with GET and return 404 if object is not found.
        POST method allowed
        """
        node = Node.objects.get(pk=1)
        node_slug = node.slug
        node_id = node.id
        node2 = Node.objects.get(pk=2)
        node2_slug = node2.slug
        fake_node_slug = "idontexist"

        url = reverse('api_node_comments', args=[node_slug])
        url2 = reverse('api_node_comments', args=[node2_slug])
        wrong_url = reverse('api_node_comments', args=[fake_node_slug])

        # api_node_comments

        # GET

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        # add two comments to two different nodes for testing purposes
        Comment.objects.create(node=node, user_id=1, text='node 1, user 1')
        Comment.objects.create(node=node2, user_id=2, text='node 2, user 2')

        # ensure the right amounts of comments are loaded!
        response = self.client.get(url)
        self.assertEqual(len(json.loads(response.content)), Comment.objects.filter(node=node).count())

        response = self.client.get(wrong_url)
        self.assertEqual(response.status_code, 404)

        # POST
        self.client.login(username='admin', password='tester')
        good_post_data = {"text": "test_comment"}
        bad_post_data = {"node": 100, "text": "test_comment_bad", "user": 2}

        # wrong slug -- 404
        response = self.client.post(wrong_url, good_post_data)
        self.assertEqual(response.status_code, 404)

        # correct POST data and correct slug -- 201
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # POST 201 - ensure additional post data "user" and "node" are ignored
        response = self.client.post(url, bad_post_data)
        self.assertEqual(response.status_code, 201)
        comment_dict = json.loads(response.content)
        self.assertEqual(comment_dict['user'], 1)
        self.assertEqual(comment_dict['node'], 1)
        self.assertEqual(comment_dict['text'], "test_comment_bad")

        # POSTing a comment for node2
        response = self.client.post(url2, good_post_data)
        self.assertEqual(response.status_code, 201)

        # GET should not return both the comments inserted above
        response = self.client.get(url)
        comments = json.loads(response.content)
        node_comments_count = Comment.objects.filter(node_id=node_id).count()
        self.assertEqual(node_comments_count, len(comments))

        # Comments not allowed on layer
        node.layer.participation_settings.comments_allowed = False
        node.layer.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 400)
        node.layer.participation_settings.comments_allowed = True
        node.layer.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # Comments not allowed on node
        node.participation_settings.comments_allowed = False
        node.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 400)
        node.participation_settings.comments_allowed = True
        node.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # User not allowed -- 403
        self.client.logout()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 403)

    def test_ratings_api(self):
        """
        Ratings endpoint should be reachable only with POST and return 404 if object is not found.
        """
        node = Node.objects.get(pk=1)
        node_slug = node.slug
        fake_node_slug = "idontexist"

        url = reverse('api_node_ratings', args=[node_slug])
        wrong_url = reverse('api_node_ratings', args=[fake_node_slug])

        # api_node_ratings

        # GET not allowed -- 405

        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)

        # POST
        self.client.login(username='admin', password='tester')
        good_post_data = {"value": "5"}

        # wrong slug -- 404
        response = self.client.post(wrong_url, good_post_data)
        self.assertEqual(response.status_code, 404)

        # wrong POST data (wrong rating) -- 400
        bad_post_data = {"node": 1, "value": "12", "user": 2}
        response = self.client.post(url, bad_post_data)
        self.assertEqual(response.status_code, 400)

        # Correct  POST data and correct slug-- 201
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # POST 201
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # POST 201 - ensure additional post data "user" and "node" are ignored
        # Tested as a different user or 400 would be returned because 'value' and 'user' are unique_together
        self.client.login(username='romano', password='tester')
        bad_post_data = {"node": 100, "value": "10", "user": 2}
        response = self.client.post(url, bad_post_data)
        self.assertEqual(response.status_code, 201)
        ratings_dict = json.loads(response.content)
        self.assertEqual(ratings_dict['user'], 4)
        self.assertEqual(ratings_dict['node'], 1)
        self.assertEqual(ratings_dict['value'], 10)

        # Rating not allowed on layer
        # Tested as a different user or 400 would be returned because 'value' and 'user' are unique_together
        self.client.logout()
        self.client.login(username='pisano', password='tester')
        node.layer.participation_settings.rating_allowed = False
        node.layer.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 400)
        node.layer.participation_settings.rating_allowed = True
        node.layer.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # Rating not allowed on node
        # Tested as a different user or 400 would be returned because 'value' and 'user' are unique_together
        self.client.logout()
        self.client.login(username='viterbese', password='tester')
        node.participation_settings.rating_allowed = False
        node.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 400)
        node.participation_settings.rating_allowed = True
        node.participation_settings.save()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)

        # User not allowed -- 403
        self.client.logout()
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 403)

    def test_votes_api(self):
        """
        Vote endpoint should be reachable only with POST and return 404 if object is not found.
        """
        node = Node.objects.get(pk=1)
        node_slug = node.slug
        fake_node_slug = "idontexist"
        url = reverse('api_node_votes', args=[node_slug])
        wrong_url = reverse('api_node_votes', args=[fake_node_slug])
        # GET not allowed -- 405
        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)
        self.client.login(username='admin', password='tester')
        good_post_data = {"vote": "1"}
        # wrong slug -- 404
        response = self.client.post(wrong_url, good_post_data)
        self.assertEqual(response.status_code, 404)
        # wrong POST data (wrong vote) -- 400
        bad_post_data = {"vote": "3"}
        response = self.client.post(url, bad_post_data)
        self.assertEqual(response.status_code, 400)
        # Correct  POST data and correct slug -- 201
        response = self.client.post(url, good_post_data)
        self.assertEqual(response.status_code, 201)
        # same result as before even if trying to change node and user
        response = self.client.post(reverse('api_node_votes', args=['pomezia']), {"node": 100, "vote": "1", "user": 3})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').count(), 1)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').first().vote, 1)

    def test_votes_api_delete_ok(self):
        self.client.login(username='admin', password='tester')
        url = reverse('api_node_votes', args=['pomezia'])
        # CREATE
        response = self.client.post(url, {"vote": "1"})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').count(), 1)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').first().vote, 1)
        # DELETE
        response = self.client.delete(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').count(), 0)

    def test_votes_api_delete_fail(self):
        self.client.login(username='admin', password='tester')
        url = reverse('api_node_votes', args=['pomezia'])
        # DELETE FAILS
        response = self.client.delete(url)
        self.assertEqual(response.status_code, 400)

    def test_votes_api_new_vote_changes_old_one(self):
        self.client.login(username='admin', password='tester')
        url = reverse('api_node_votes', args=['pomezia'])
        # 201
        response = self.client.post(url, {"vote": "1"})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').count(), 1)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').first().vote, 1)
        # POST 201 - repeating the same vote deletes old vote
        response = self.client.post(url, {"vote": "-1"})
        self.assertEqual(response.status_code, 201)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').count(), 1)
        self.assertEqual(Vote.objects.filter(node__slug='pomezia', user__username='admin').first().vote, -1)

    def test_votes_api_post(self):
        # POST 201 - ensure additional post data "user" and "node" are ignored
        self.client.login(username='admin', password='tester')
        response = self.client.post(reverse('api_node_votes', args=['eigenlab']), {"vote": "1"})
        self.assertEqual(response.status_code, 201)
        votes_dict = json.loads(response.content)
        self.assertEqual(votes_dict['user'], 1)
        self.assertEqual(votes_dict['node'], 2)
        self.assertEqual(votes_dict['vote'], 1)

    def test_votes_api_not_allowed_on_layer(self):
        self.client.login(username='admin', password='tester')
        url = reverse('api_node_votes', args=['tulug'])
        node = Node.objects.get(slug='tulug')
        # Voting not allowed on layer
        node.layer.participation_settings.voting_allowed = False
        node.layer.participation_settings.save()
        response = self.client.post(url, {"vote": "1"})
        self.assertEqual(response.status_code, 400)
        node.layer.participation_settings.voting_allowed = True
        node.layer.participation_settings.save()
        response = self.client.post(url, {"vote": "1"})
        self.assertEqual(response.status_code, 201)

    def test_votes_api_not_allowed_on_node(self):
        self.client.login(username='admin', password='tester')
        url = reverse('api_node_votes', args=['tulug'])
        node = Node.objects.get(slug='tulug')
        # Voting not allowed on node
        node.participation_settings.voting_allowed = False
        node.participation_settings.save()
        response = self.client.post(url, {"vote": "1"})
        self.assertEqual(response.status_code, 400)
        node.participation_settings.voting_allowed = True
        node.participation_settings.save()
        response = self.client.post(url, {"vote": "1"})
        self.assertEqual(response.status_code, 201)

    def test_votes_api_post_not_allowed_by_user(self):
        # User not allowed -- 403
        response = self.client.post(reverse('api_node_votes', args=['tulug']), {"vote": "1"})
        self.assertEqual(response.status_code, 403)

    def test_layer_comments_api(self):
        """
        Layer comments endpoint should be reachable only with GET and return 404 if object is not found.
        """
        layer = Layer.objects.get(pk=1)
        layer_slug = layer.slug
        fake_layer_slug = "idontexist"
        url = reverse('api_layer_nodes_comments', args=[layer_slug])
        wrong_url = reverse('api_layer_nodes_comments', args=[fake_layer_slug])

        # api_layer_nodes_comments

        # GET
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        response = self.client.get(wrong_url)
        self.assertEqual(response.status_code, 404)

        # POST not allowed -- 405
        self.client.login(username='admin', password='tester')
        response = self.client.post(url)
        self.assertEqual(response.status_code, 405)

    def test_layer_participation_api(self):
        """
        Layer participation endpoint should be reachable only with GET and return 404 if object is not found.
        """
        layer = Layer.objects.get(pk=1)
        layer_slug = layer.slug
        fake_layer_slug = "idontexist"
        url = reverse('api_layer_nodes_participation', args=[layer_slug])
        wrong_url = reverse('api_layer_nodes_participation', args=[fake_layer_slug])

        # api_layer_nodes_participation

        # GET
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        response = self.client.get(wrong_url)
        self.assertEqual(response.status_code, 404)

        # POST not allowed -- 405
        self.client.login(username='admin', password='tester')
        response = self.client.post(url)
        self.assertEqual(response.status_code, 405)

    def test_node_participation_api(self):
        """
        Participation endpoint should be reachable only with GET and return 404 if object is not found.
        """
        node = Node.objects.get(pk=1)
        node_slug = node.slug
        fake_node_slug = "idontexist"
        url = reverse('api_node_participation', args=[node_slug])
        wrong_url = reverse('api_node_participation', args=[fake_node_slug])

        # api_node_participation

        # GET
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        participation_dict = json.loads(response.content)
        likes_count = node.noderatingcount.likes
        dislikes_count = node.noderatingcount.dislikes
        comment_count = node.noderatingcount.comment_count
        rating_count = node.noderatingcount.rating_count
        rating_avg = node.noderatingcount.rating_avg
        self.assertEqual(participation_dict['participation']['likes'], likes_count)
        self.assertEqual(participation_dict['participation']['dislikes'], dislikes_count)
        self.assertEqual(participation_dict['participation']['comment_count'], comment_count)
        self.assertEqual(participation_dict['participation']['rating_count'], rating_count)
        self.assertEqual(participation_dict['participation']['rating_avg'], rating_avg)
        response = self.client.get(wrong_url)
        self.assertEqual(response.status_code, 404)

        # POST not allowed -- 405
        self.client.login(username='admin', password='tester')
        response = self.client.post(url)
        self.assertEqual(response.status_code, 405)

    def test_voted_on_node(self):
        node = Node.objects.get(pk=1)
        url = reverse('api_node_details', args=[node.slug])

        # logged out expects False
        self.client.logout()
        response = self.client.get(url)
        self.assertEqual(response.data['relationships']['voted'], False)

        # logged in not voted yet expects False
        self.client.login(username='admin', password='tester')
        response = self.client.get(url)
        self.assertEqual(response.data['relationships']['voted'], False)

        # has already liked expects 1
        v = Vote.objects.create(node_id=node.id, user_id=1, vote=1)
        response = self.client.get(url)
        self.assertEqual(response.data['relationships']['voted'], 1)

        # has already disliked expects -1
        v.delete()
        v = Vote.objects.create(node_id=node.id, user_id=1, vote=-1)
        response = self.client.get(url)
        self.assertEqual(response.data['relationships']['voted'], -1)

    def test_voting_allowed(self):
        node = Node.objects.last()
        self.assertTrue(node.voting_allowed)
        # disable voting on layer
        node.layer.participation_settings.voting_allowed = False
        node.layer.participation_settings.save()
        self.assertFalse(node.voting_allowed)
        # re-enable on layer
        node.layer.participation_settings.voting_allowed = True
        node.layer.participation_settings.save()
        self.assertTrue(node.voting_allowed)
        # disable on node
        node.participation_settings.voting_allowed = False
        node.participation_settings.save()
        self.assertFalse(node.voting_allowed)

    def test_rating_allowed(self):
        node = Node.objects.last()
        self.assertTrue(node.rating_allowed)
        # disable rating on layer
        node.layer.participation_settings.rating_allowed = False
        node.layer.participation_settings.save()
        self.assertFalse(node.rating_allowed)
        # re-enable on layer
        node.layer.participation_settings.rating_allowed = True
        node.layer.participation_settings.save()
        self.assertTrue(node.rating_allowed)
        # disable on node
        node.participation_settings.rating_allowed = False
        node.participation_settings.save()
        self.assertFalse(node.rating_allowed)

    def test_comments_allowed(self):
        node = Node.objects.last()
        self.assertTrue(node.comments_allowed)
        # disable comments on layer
        node.layer.participation_settings.comments_allowed = False
        node.layer.participation_settings.save()
        self.assertFalse(node.comments_allowed)
        # re-enable on layer
        node.layer.participation_settings.comments_allowed = True
        node.layer.participation_settings.save()
        self.assertTrue(node.comments_allowed)
        # disable on node
        node.participation_settings.comments_allowed = False
        node.participation_settings.save()
        self.assertFalse(node.comments_allowed)



from celery import task
from django.core import management


@task
def import_old_nodeshot(*args, **kwargs):
    """
    runs "python manage.py import_old_nodeshot"
    """
    management.call_command('import_old_nodeshot', *args, **kwargs)



from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model


class EmailBackend(ModelBackend):
    """
    Authenticates against user email.
    """

    def authenticate(self, username=None, password=None, **kwargs):
        usermodel = get_user_model()
        try:
            user = usermodel.objects.get(email=username)
            if user.check_password(password):
                return user
        except usermodel.DoesNotExist:
            return None



# -*- coding: utf-8 -*-
from django.utils.safestring import mark_safe
from django.utils.html import escape, conditional_escape
from django.utils.encoding import force_unicode
from django.forms.widgets import ClearableFileInput, CheckboxInput
from django.utils.translation import ugettext as _


class AdvancedFileInput(ClearableFileInput):
    """
    File Input Widget
    """

    def __init__(self, *args, **kwargs):

        self.url_length = kwargs.pop('url_length',30)
        self.preview = kwargs.pop('preview',True)
        self.image_width = kwargs.pop('image_width',200)
        super(AdvancedFileInput, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None,):

        substitutions = {
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        template = u'%(input)s'

        substitutions['input'] = super(ClearableFileInput, self).render(name, value, attrs)

        if value and hasattr(value, "url"):

            template = self.template_with_initial
            if self.preview:
                substitutions['initial'] = (u'<a href="{0}">{1}</a><br /><br />\
                <a href="{0}" target="_blank"><img src="{0}" alt="" width="{2}" /></a><br /><br />'.format
                    (escape(value.url),'...'+escape(force_unicode(value))[-self.url_length:],
                     self.image_width))
            else:
                substitutions['initial'] = (u'<a href="{0}">{1}</a>'.format
                    (escape(value.url),'...'+escape(force_unicode(value))[-self.url_length:]))
            if not self.is_required:
                checkbox_name = self.clear_checkbox_name(name)
                checkbox_id = self.clear_checkbox_id(checkbox_name)
                substitutions['clear_checkbox_name'] = conditional_escape(checkbox_name)
                substitutions['clear_checkbox_id'] = conditional_escape(checkbox_id)
                substitutions['clear'] = CheckboxInput().render(checkbox_name, False, attrs={'id': checkbox_id})
                substitutions['clear_template'] = self.template_with_clear % substitutions

        return mark_safe(template % substitutions)


# originally taken from django-colorful but adapted

from django.conf import settings
from django.forms.widgets import TextInput
from django.utils.safestring import SafeUnicode

try:
    url = settings.STATIC_URL
except AttributeError:
    try:
        url = settings.MEDIA_URL
    except AttributeError:
        url = ''

class ColorFieldWidget(TextInput):
    class Media:
        css = {
            'all': ("%scolorful/spectrum.css" % url,)
        }
        js  = ("%scolorful/spectrum.js" % url,)

    input_type = 'color'

    def render_script(self, id):
        return u'''<script type="text/javascript">
                    (function($){
                        $(document).ready(function(){
                            $('#%s').each(function(i, elm){
                                // Make sure html5 color element is not replaced
                                if(elm.type != 'color'){
                                    $(elm).spectrum({
                                        showInput: true,
                                        showInitial: true,
                                        cancelText: '%s',
                                        chooseText: '%s'
                                    });
                                }
                            });
                        });
                    })('django' in window ? django.jQuery : jQuery);
                </script>
                ''' % (id, _('cancel'), _('ok'))

    def render(self, name, value, attrs={}):
        if 'id' not in attrs:
            attrs['id'] = "#id_%s" % name
        render = super(ColorFieldWidget, self).render(name, value, attrs)
        return SafeUnicode(u"%s%s" % (render, self.render_script(attrs['id'])))
